package allrounder;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;

/**
 * Manages all the directory paths
 * @author klaue
 *
 */
public class DirHandler {
	/**
	 * Gets the application path as file
	 * @return The path of YiffIndex as File
	 */
	public static File getAppDir() {
		try {
			return new File(URLDecoder.decode(DirHandler.class.getResource("/").getPath(), "UTF-8"));
		} catch (UnsupportedEncodingException e) {
			return null; // will never happen.. Java supports UTF8
		}
	}
	
	/**
	 * Gets the application path
	 * @return The path of YiffIndex
	 */
	public static String getAppPath() {
		return getAppDir().getAbsolutePath();
	}
	
	/**
	 * Gets the directory where Files that need write access are (should be)
	 * @return The Directory as a File
	 */
	public static File getAppDataDir() {
		File f = new File(System.getProperty("user.home"), ".YiffIndex");
		if (!f.exists()) {
			f.mkdirs();
			// copy the db scripts
			try {
				File temp = new File(getAppDir(), "db");
				copy(new File(temp, "databasemanager.bat"), new File(f, "databasemanager.bat"));
				copy(new File(temp, "databasemanager.sh"), new File(f, "databasemanager.sh"));
				copy(new File(temp, "startserver.bat"), new File(f, "startserver.bat"));
				copy(new File(temp, "startserver.sh"), new File(f, "startserver.sh"));
			} catch (Exception e) {
				e.printStackTrace();
				// do nothing. the scripts are missing, boo-hoo
			}
		}
		
		return f;
	}
	
	/**
	 * Gets the directory where Files that need write access are (should be)
	 * @return The Directory as a String
	 */
	public static String getAppDataPath() {
		return getAppDataDir().getAbsolutePath();
	}
	
	/**
	 * Returns the directory of the passed class (mainly for plugins)
	 * @param cl The class of which you want the directory 
	 * @return The directory of the class
	 */
	public static File getDirOfClass(Class<?> cl) {
		try {
			return new File(URLDecoder.decode(cl.getResource("").getPath(), "UTF-8"));
		} catch (UnsupportedEncodingException e) {
			return null; // will never happen.. Java supports UTF8
		}
	}
	
	/**
	 * Returns the path of the directory of the passed class (mainly for plugins)
	 * @param cl The class of which you want the directory 
	 * @return The path of the directory of the class
	 */
	public static String getPathOfClass(Class<?> cl) {
		return getDirOfClass(cl).getAbsolutePath();
	}
	
	private static void copy(File src, File dst) throws IOException {
		InputStream in = new FileInputStream(src);
		OutputStream out = new FileOutputStream(dst);
		// Transfer bytes from in to out
		byte[] buf = new byte[1024];
		int len;
		while ((len = in.read(buf)) > 0) {
			out.write(buf, 0, len);
		}
		in.close();
		out.close();
	} 
}
