package allrounder;

import java.io.Externalizable;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.TreeSet;

import backup.online.json.JSONArray;
import backup.online.json.JSONObject;

public class TaggedImage implements Comparable<TaggedImage>, Externalizable {
	static final long serialVersionUID = 8673770392938824493L;
	
	public int ID = 0;
	public File path = null;
	public int height = 0;
	public int width = 0;
	public boolean yiff = false;
	
	public String hash = null;
	public boolean changed = false;
	public int headcount = 0;
	public TreeSet<Integer> spezies = new TreeSet<Integer>();
	public TreeSet<String> artists = new TreeSet<String>();
	public TreeSet<Integer> gender = new TreeSet<Integer>();
	public TreeSet<String> other = new TreeSet<String>();
	public TreeSet<Integer> fetish = new TreeSet<Integer>();
	
	public boolean isEmpty() {
		return (!(this.headcount != 0 || this.spezies.size() != 0 || this.artists.size() != 0 ||
				this.gender.size() != 0 || this.other.size() != 0 || this.fetish.size() != 0 ||
				this.yiff == true));
	}
	
	public int compareTo(TaggedImage o) {
		if (equals(o)) return 0;
		if (this.path != null && o.path != null) {
			return this.path.compareTo(o.path);
		} else if (this.path == null && o.path == null) {
			return this.hash.compareTo(o.hash);
		} else if (this.path == null && o.path != null) {
			return -1;
		}
		return 1;
	}
	
	public void writeExternal(ObjectOutput stream) throws IOException {
		// own implementation to not write the path, id and changed
		stream.writeUTF(this.hash);
		stream.writeBoolean(this.yiff);
		stream.writeInt(this.headcount);
		stream.writeObject(this.spezies);
		stream.writeObject(this.artists);
		stream.writeObject(this.gender);
		stream.writeObject(this.other);
		stream.writeObject(this.fetish);
		stream.writeInt(this.width);
		stream.writeInt(this.height);
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((this.artists == null) ? 0 : this.artists.hashCode());
		result = prime * result + ((this.fetish == null) ? 0 : this.fetish.hashCode());
		result = prime * result + ((this.gender == null) ? 0 : this.gender.hashCode());
		result = prime * result + ((this.hash == null) ? 0 : this.hash.hashCode());
		result = prime * result + this.headcount;
		result = prime * result + ((this.other == null) ? 0 : this.other.hashCode());
		result = prime * result + ((this.path == null) ? 0 : this.path.hashCode());
		result = prime * result + ((this.spezies == null) ? 0 : this.spezies.hashCode());
		result = prime * result + (this.yiff ? 1231 : 1237);
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof TaggedImage)) return false;
		TaggedImage img = (TaggedImage)obj;
		try {
			if (((img.hash == null && this.hash == null) || this.hash.equals(img.hash)) &&
					((img.path == null && this.path == null) || this.path.equals(img.path)) && this.yiff == img.yiff &&
					this.headcount == img.headcount && this.spezies.equals(img.spezies) &&
					this.artists.equals(img.artists) && this.gender.equals(img.gender) &&
					this.other.equals(img.other) && this.fetish.equals(img.fetish)) return true;
		} catch (NullPointerException e) {
			// hash.equals or path.equals could throw this
		}
		return false;
	}

	public JSONObject toJSONObject() {
		JSONArray aSpezies = new JSONArray();
		for (Integer i : this.spezies) aSpezies.add(i);
		JSONArray aArtist = new JSONArray();
		for (String s : this.artists) aArtist.add(s);
		JSONArray aGender = new JSONArray();
		for (Integer i : this.gender) aGender.add(i);
		JSONArray aOther = new JSONArray();
		for (String s : this.other) aOther.add(s);
		JSONArray aFetish = new JSONArray();
		for (Integer i : this.fetish) aFetish.add(i);
		
		JSONObject obj = new JSONObject();
		obj.put("hash", this.hash);
		obj.put("yiff", this.yiff);
		obj.put("headcount", this.headcount);
		obj.put("spezies", aSpezies);
		obj.put("artists", aArtist);
		obj.put("gender", aGender);
		obj.put("other", aOther);
		obj.put("fetish", aFetish);
		obj.put("width", this.width);
		obj.put("height", this.height);
		return obj;
	}
	
	@SuppressWarnings("unchecked")
	public void readExternal(ObjectInput stream) throws IOException {
		try {
			this.hash = stream.readUTF();
			this.yiff = stream.readBoolean();
			this.headcount = stream.readInt();
			this.spezies = (TreeSet<Integer>)stream.readObject();
			this.artists = (TreeSet<String>)stream.readObject();
			this.gender = (TreeSet<Integer>)stream.readObject();
			this.other = (TreeSet<String>)stream.readObject();
			this.fetish = (TreeSet<Integer>)stream.readObject();
			this.width = stream.readInt();
			this.height = stream.readInt();
			
			// any imported pic will be set to changed = false
			this.changed = false;
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void fromJSONObject(JSONObject obj) {
		JSONArray aSpezies = (JSONArray)obj.get("spezies");
		JSONArray aArtist = (JSONArray)obj.get("artists");
		JSONArray aGender = (JSONArray)obj.get("gender");
		JSONArray aOther = (JSONArray)obj.get("other");
		JSONArray aFetish = (JSONArray)obj.get("fetish");
		
		this.hash = (String)obj.get("hash");
		this.yiff = (Boolean)obj.get("yiff");
		// parser makes int to a long..
		this.headcount = ((Long)obj.get("headcount")).intValue();
		for (int i = 0; i < aSpezies.size(); ++i) this.spezies.add(((Long)aSpezies.get(i)).intValue());
		for (int i = 0; i < aArtist.size(); ++i) this.artists.add((String)aArtist.get(i));
		for (int i = 0; i < aGender.size(); ++i) this.gender.add(((Long)aGender.get(i)).intValue());
		for (int i = 0; i < aOther.size(); ++i) this.other.add((String)aOther.get(i));
		for (int i = 0; i < aFetish.size(); ++i) this.fetish.add(((Long)aFetish.get(i)).intValue());

		this.width = ((Long)obj.get("width")).intValue();
		this.height = ((Long)obj.get("height")).intValue();
		// any imported pic will be set to changed = false
		this.changed = false;
	}
	
	@Override
	public String toString() {
		StringBuilder str = new StringBuilder();
		str.append("TaggedImage: ");
		str.append("ID=" + this.ID + " ");
		str.append("path=" + this.path + " ");
		str.append("height=" + this.height + " ");
		str.append("width=" + this.width + " ");

		str.append("hash=" + this.hash + " ");
		str.append("yiff=" + this.yiff + " ");
		str.append("headcount=" + this.headcount + " ");
		str.append("spezies=" + this.spezies + " ");
		str.append("artists=" + this.artists + " ");
		str.append("gender=" + this.gender + " ");
		str.append("other=" + this.other + " ");
		str.append("fetish=" + this.fetish + " ");
		str.append("yiff=" + this.changed);
		return str.toString();
	}
}
