package allrounder.plugin;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.TreeMap;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import allrounder.DbHandler;
import allrounder.StringLib;

public class PluginBrowser extends JDialog implements ActionListener {
	StringLib lib = StringLib.getInstance();
	PluginLoader loader = PluginLoader.getInstance();
	JLabel lblAviablePlugins = new JLabel(this.lib.getValue("pluginbrowser.AviablePlugins"));
	JTable tblPlugins = null;
	JButton btnSave = new JButton(this.lib.getValue("pluginbrowser.save"));
	JButton btnAbort = new JButton(this.lib.getValue("pluginbrowser.abort"));
	
	TreeMap<String, YiffIndexPlugin> allPlugins;
	TreeMap<String, YiffIndexPlugin> loadedPlugins;
	
	public PluginBrowser(JFrame f) {
		this.allPlugins = PluginLoader.getAllAviablePlugins();
		this.loadedPlugins = this.loader.getLoadedPlugins();
		
		setTitle(this.lib.getValue("pluginbrowser.title"));
		//setModal(true);
		setSize(800, 300);
		setLocationRelativeTo(f);
		((JPanel)this.getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		//tabelle
		buildTable();
		JScrollPane scrPane = new JScrollPane(this.tblPlugins);

		this.btnSave.setActionCommand("SAVE");
		this.btnSave.addActionListener(this);
		this.btnAbort.setActionCommand("ABORT");
		this.btnAbort.addActionListener(this);
		
		//column / row
		double[][] layout = {{this.btnSave.getPreferredSize().width, 10, this.btnAbort.getPreferredSize().width, TableLayoutConstants.FILL},
				{this.lblAviablePlugins.getPreferredSize().height, 10, TableLayoutConstants.FILL, 10, this.btnSave.getPreferredSize().height}};
		this.setLayout(new TableLayout(layout));
		
		this.add(this.lblAviablePlugins, "0, 0, 3, 0");
		this.add(scrPane, "0, 2, 3, 2");
		this.add(this.btnSave, "0, 4");
		this.add(this.btnAbort, "2, 4");
		setVisible(true);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}
	
	private void buildTable() {
		String[] columnNames = {"",
				this.lib.getValue("pluginbrowser.name"),
				this.lib.getValue("pluginbrowser.descr"),
				this.lib.getValue("pluginbrowser.class")};
		Object[][] data = new Object[this.allPlugins.size()][columnNames.length];
		
		Iterator<String> iter = this.allPlugins.keySet().iterator();
		int i = 0;
		while (iter.hasNext()) {
			String sClassPath = iter.next();
			data[i][0] = this.loadedPlugins.containsKey(sClassPath);
			data[i][1] = this.allPlugins.get(sClassPath).getName();
			data[i][2] = this.allPlugins.get(sClassPath).getDescription();
			data[i][3] = sClassPath;
			++i;
		}
		
		int[] wrapindices = {2};
		final WrappingRenderer tatcrRenderer = new WrappingRenderer(wrapindices);
		
		TableModel mdl = new DefaultTableModel(data, columnNames) {
			@Override
			public boolean isCellEditable(int row, int column) {
				return (column == 0);
			}
			
			@Override
			public Class<?> getColumnClass(int c) {
				if (getDataVector().size() == 0) return super.getColumnClass(c);
		        return getValueAt(0, c).getClass();
		    }
		};
		
		this.tblPlugins = new JTable(mdl){
		    @Override
			public TableCellRenderer getCellRenderer(int row, int column) {
		        if ((convertColumnIndexToModel(column) == 2)) {
		            return tatcrRenderer;
		        }
		        return super.getCellRenderer(row, column);
		    }
		};
		TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(); 
		this.tblPlugins.setRowSorter( sorter );
        sorter.setModel( this.tblPlugins.getModel() );
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equalsIgnoreCase("ABORT")) {
			setVisible(false);
			dispose();
		} else if (arg0.getActionCommand().equalsIgnoreCase("SAVE")) {
			//tblSource.getValueAt(row, tblSource.convertColumnIndexToView(7))
			for (int i = 0; i < this.tblPlugins.getRowCount(); ++i) {
				boolean bActivated = (Boolean)this.tblPlugins.getValueAt(i, this.tblPlugins.convertColumnIndexToView(0));
				String sClassPath = (String)this.tblPlugins.getValueAt(i, this.tblPlugins.convertColumnIndexToView(3));
				if (this.loadedPlugins.containsKey(sClassPath) != bActivated) {
					// add plugin to loaded ones or remove it
					if (bActivated) DbHandler.getInstance().addPlugin(sClassPath);
					if (!bActivated) DbHandler.getInstance().removePlugin(sClassPath);
				}
			}
			this.loader.reloadPluginsFromDB();
			setVisible(false);
			dispose();
		}
	}
}
