package allrounder.plugin;

import java.awt.Component;
import java.awt.Rectangle;

import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.TableCellRenderer;

/**
 * A TableCellRenderer which helps JTable display multiline text
 * Only supposed to be used with Strings as TableData!
 * @author Klaue
 *
 */
class WrappingRenderer implements TableCellRenderer {
	// for people who would need it: if you want to use HTML (for artifical linebreaks through <br> or so) use
	// JEditorPane instead of JTextArea and add tA.setContentType("text/html"); to the ctor
	// Warning, this results in a character-based linewrap
	JTextArea tA;
	int[] columns;

	/**
	 * This is the constructor for TextAreaTableCellRenderer
	 * @param columns All the colums (in the model) you plan to use this with or null for only one column
	 */
	public WrappingRenderer(int[] columns) {
		this.columns = columns;
		this.tA = new JTextArea();
		this.tA.setLineWrap(true);
		this.tA.setWrapStyleWord(true);
	}

	public Component getTableCellRendererComponent(JTable table, Object obj,
			boolean isSelected, boolean hasFocus, int row, int column) {

		try {
			this.tA.setText((String) obj);
		} catch (ClassCastException e) {
			// should never happen if user used strings as he should
			this.tA.setText(obj.toString());
			System.err.println(e.getMessage()); // just to bug the faulty user :D
		}

		Rectangle r = table.getCellRect(row, column, true);
		this.tA.setSize(r.width, r.height);

		int height_wanted = (int) this.tA.getPreferredScrollableViewportSize()
				.getHeight();
		if (height_wanted != table.getRowHeight(row)) {
			if (this.columns == null || this.columns.length == 1) {
				table.setRowHeight(row, height_wanted);
			} else {
				// check if the wanted height is less than the height of the
				// other resizable cells
				boolean setNewHeight = false;
				for (int columnIndex : this.columns) {
					columnIndex = table.convertColumnIndexToView(columnIndex);
					if (column == columnIndex) continue;
					JTextArea taOther = new JTextArea();
					taOther.setLineWrap(true);
					taOther.setWrapStyleWord(true);
					try {
						taOther.setText((String) table.getValueAt(row, columnIndex));
					} catch (ClassCastException e) {
						// should never happen if user used strings as he should
						taOther.setText(table.getValueAt(row,columnIndex).toString());
						System.err.println(e.getMessage()); // just to bug the faulty user :D
					}
					Rectangle r2 = table.getCellRect(row, columnIndex, true);
					taOther.setSize(r2.width, r2.height);
					
					if (height_wanted >= (int) taOther.getPreferredScrollableViewportSize().getHeight()) {
						setNewHeight = true;
						break;
					}
				}
				if (setNewHeight) table.setRowHeight(row, height_wanted);
			}
		}
		return this.tA;
	}
}