package backup.online;

import index.IndexPanel;

import java.awt.Cursor;
import java.awt.Toolkit;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;

import javax.swing.JOptionPane;
import javax.swing.ProgressMonitor;
import javax.swing.SwingWorker;

import klaue.DownloadHelper.Downloader;
import allrounder.DbHandler;
import allrounder.DirHandler;
import allrounder.StringLib;
import allrounder.TaggedImage;
import allrounder.md5.Hasher;
import backup.online.json.JSONArray;
import backup.online.json.JSONObject;
import backup.online.json.JSONValue;

import com.sun.jimi.core.Jimi;
import com.sun.jimi.core.JimiException;

public class ServerCommunication {
	static public final String PHP_URL = "https://double-helix.industries/applications/yiffindex/YiffIndex.php";
	
	public static void doUpload(ArrayList<TaggedImage> images, IndexPanel panel) {
		final StringLib lib = StringLib.getInstance();
		
		// "convert" to final to get them to work in the anonymous inner class
		final IndexPanel pnl = panel;
		
		pnl.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		// if the given parameter is null, upload /ALL/ images from the local DB to the online DB
		if (images == null) {
			images = new ArrayList<TaggedImage>();
			ArrayList<Integer> ids = DbHandler.getInstance().getAllImageDataIds();
			for (int i : ids) {
				images.add(DbHandler.getInstance().getIncompleteImageFromImageDataId(i));
			}
		}
		
		// change the map to an ArrayList containing JSONArrays of 100 JSONObjects
		ArrayList<JSONArray> alJSONArrays = new ArrayList<JSONArray>();
		alJSONArrays.add(new JSONArray());
		int numOfImgs = 0;
		for (TaggedImage img : images) {
			if (!img.changed) continue;
			alJSONArrays.get(alJSONArrays.size() - 1).add(img.toJSONObject());
			++numOfImgs;
			
			if (alJSONArrays.get(alJSONArrays.size() - 1).size() >= 100) {
				alJSONArrays.add(new JSONArray());
			}
		}
		
		//preparations complete
		try {
			if(!checkVersion()) {
				// falsche version
				JOptionPane.showMessageDialog(pnl, lib.getValue("backup_online.generic.error.wrongVers.text"),
						lib.getValue("backup_online.generic.error.wrongVers.title"), JOptionPane.ERROR_MESSAGE);
				pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				return;
			}
		} catch (Exception e) {
			// fehler
			e.printStackTrace();
			JOptionPane.showMessageDialog(pnl, lib.getValue("backup_online.upload.error.generic.text"),
					lib.getValue("backup_online.upload.error.generic.title"), JOptionPane.ERROR_MESSAGE);
			pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			return;
		}
			
		UploadWorker worker = new UploadWorker(alJSONArrays, pnl, numOfImgs);
		worker.execute();
	}
	
	public static void doDownload(TreeMap<String, ArrayList<File>> hashesToDownload, IndexPanel panel) {
		final StringLib lib = StringLib.getInstance();
		if (hashesToDownload == null) hashesToDownload = new TreeMap<String, ArrayList<File>>();
		
		// "convert" to final to get them to work in the anonymous inner class
		final IndexPanel pnl = panel;
		
		pnl.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		try {
			Boolean checkVers = checkVersion();
			if (checkVers == null) {
				// server offline
				JOptionPane.showMessageDialog(pnl, lib.getValue("backup_online.generic.error.ServerOffline.text"),
						lib.getValue("backup_online.generic.error.ServerOffline.title"), JOptionPane.ERROR_MESSAGE);
				pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				return;
			} else if (!checkVers.booleanValue()) {
				// falsche version
				JOptionPane.showMessageDialog(pnl, lib.getValue("backup_online.generic.error.wrongVers.text"),
						lib.getValue("backup_online.generic.error.wrongVers.title"), JOptionPane.ERROR_MESSAGE);
				pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				return;
			}
		} catch (Exception e) {
			// fehler
			e.printStackTrace();
			JOptionPane.showMessageDialog(pnl, lib.getValue("backup_online.upload.error.generic.text"),
					lib.getValue("backup_online.upload.error.generic.title"), JOptionPane.ERROR_MESSAGE);
			pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			return;
		}
		
		// check if all or just untagged images are to be updated
		boolean bImportAll;
		if (hashesToDownload.size() != 1) {
			int allImport = JOptionPane.showConfirmDialog(null, lib.getValue("backup_online.download.allImport.text"), lib.getValue("backup_online.download.allImport.title"), JOptionPane.YES_NO_CANCEL_OPTION);
			if (allImport != JOptionPane.YES_OPTION && allImport != JOptionPane.NO_OPTION) return;
			bImportAll = (allImport == JOptionPane.YES_OPTION);
		} else {
			bImportAll = true;
		}
		
		
		
		DownloadWorker worker = new DownloadWorker(hashesToDownload, bImportAll, pnl);
		worker.execute();
	}
	
	private static Boolean checkVersion() throws IOException {
		String sPhpParams = "function=checkversion&param=" + DbHandler.getInstance().getGeneral("Version");
		//String sAnswer = "";
		ArrayList<String> al = Downloader.sendPOST(new URL(PHP_URL), sPhpParams, null, "YiffIndex", 60000, 3);
		if (al == null) return null;
		for (String s : al) System.out.println("checkversion: " + s);
		return Boolean.parseBoolean(al.get(0));
	}
}

class UploadWorker extends SwingWorker<Object, Object> implements PropertyChangeListener {
	private ArrayList<JSONArray> alJSONArrays;
	private int totalAmountOfImgs;
	
	private ProgressMonitor progressMonitor;
	
	private StringLib lib = StringLib.getInstance();
	private IndexPanel pnl;
	
	public UploadWorker(ArrayList<JSONArray> alJSONArrays, IndexPanel pnl, int totalAmountOfImgs) {
		this.alJSONArrays = alJSONArrays;
		this.pnl = pnl;
		this.totalAmountOfImgs = totalAmountOfImgs;
		addPropertyChangeListener(this);
		this.progressMonitor = new ProgressMonitor(pnl,
				this.lib.getValue("backup_online.upload.progress.text"),
	            "", 0, 100);
		this.progressMonitor.setProgress(0);
	}
	
	@Override
	protected Object doInBackground() throws Exception {
		try {
			for (int i = 0; i < this.alJSONArrays.size(); ++i) {
				if (isCancelled()) {
					setProgress(100);
					return null;
				}
				JSONArray arr = this.alJSONArrays.get(i);
				if (!uploadAsFile(arr)) {
					// abbruch, irgendein fehler
					JOptionPane.showMessageDialog(this.pnl, this.lib.getValue("backup_online.upload.error.generic.text"),
							this.lib.getValue("backup_online.upload.error.generic.title"), JOptionPane.ERROR_MESSAGE);
					this.pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return null;
				}
				
				// set all successfully uploaded images to changed=false
				for (Object obj : arr) {
					DbHandler.getInstance().setImageChanged((String)((JSONObject)obj).get("hash"), false);
				}
				setProgress((int)(100*((double)(i+1)/(double)this.alJSONArrays.size())));
				//setProgress((int)(100*((double)(i+1/alJSONArrays.size()))));
			}
			this.progressMonitor.setProgress(100);
			JOptionPane.showMessageDialog(this.pnl, String.format(this.lib.getValue("backup_online.upload.finish.text"), this.totalAmountOfImgs),
					this.lib.getValue("backup_online.upload.finish.title"), JOptionPane.INFORMATION_MESSAGE);
			this.pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			return null;
		} catch (Exception e) {
			// fehler
			e.printStackTrace();
			JOptionPane.showMessageDialog(this.pnl, this.lib.getValue("backup_online.upload.error.generic.text"),
					this.lib.getValue("backup_online.upload.error.generic.title"), JOptionPane.ERROR_MESSAGE);
			this.pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			this.progressMonitor.setProgress(100);
			return null;
		}
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if ("progress" == evt.getPropertyName() ) {
			int progress = (Integer) evt.getNewValue();
			this.progressMonitor.setProgress(progress);
            String message = String.format(this.lib.getValue("backup_online.upload.progress.note"), progress);
            this.progressMonitor.setNote(message);
            if (this.progressMonitor.isCanceled() || isDone()) {
                Toolkit.getDefaultToolkit().beep();
                if (this.progressMonitor.isCanceled()) {
                    cancel(true);
                    System.gc();
                } else {
        			System.gc();
                }
                this.pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            }
		}
	}
	
	private boolean uploadAsFile(JSONArray array) throws IOException {
		File f = saveFile(array, "wackydeli.txt");
		TreeMap<String, String> mParams = new TreeMap<String, String>();
		mParams.put("function", "upload");
		mParams.put("param", f.getAbsolutePath());
		ArrayList<String> al = Downloader.sendFileUsingPOST(new URL(ServerCommunication.PHP_URL), mParams, f, null, "YiffIndex", 60000, 3);
		f.delete();
		if (al == null) return false;
		for (String s : al) System.out.println("uploadasfile: " + s);
		return Boolean.parseBoolean(al.get(0));
	}
	
	private File saveFile(JSONArray array, String filename) throws IOException {
		File file = new File(DirHandler.getAppDataDir(), filename);
		BufferedWriter output = new BufferedWriter(new FileWriter(file));
		output.write(array.toString());
		output.close();
		return file;
	}
}

class DownloadWorker extends SwingWorker<Object, Object> implements PropertyChangeListener {
	private boolean bImportAll;
	private TreeMap<String, ArrayList<File>> hashes;
	private int totalAmountOfImgs = 0;
	
	private ProgressMonitor progressMonitor;
	
	private StringLib lib = StringLib.getInstance();
	private IndexPanel pnl;
	
	public DownloadWorker(TreeMap<String, ArrayList<File>> hashes, boolean bImportAll, IndexPanel pnl) {
		this.hashes = hashes;
		this.bImportAll = bImportAll;
		this.pnl = pnl;
		addPropertyChangeListener(this);
	}
	
	@Override
	protected Object doInBackground() throws Exception {
		// get hashes
		if (this.hashes.size() == 0) {
			ArrayList<File> files = new ArrayList<File>();
	    	TreeMap<File, Boolean> folders = DbHandler.getInstance().getDirectories();
			Iterator<File> folderIter = folders.keySet().iterator();
			while (folderIter.hasNext()) {
				File folder = folderIter.next();
				for (File image : getImagesFromFolder(folder)) {
					if (!this.bImportAll) if (DbHandler.getInstance().isImageInDb(image)) continue;
					files.add(image);
				}
			}
			
			int iNumberOfImagesInFolders = files.size();
			double dProgress = 0;
			
			this.progressMonitor = new ProgressMonitor(this.pnl,
					this.lib.getValue("backup_online.download.progress.text"),
		            "", 0, 100);
			this.progressMonitor.setProgress(0);
			for (File image : files) {
				if (!image.exists()) continue;
				if (isCancelled()) {
					this.pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return null;
				}
				String hash = Hasher.imageToHash(image);
				if (this.hashes.get(hash) != null) {
					this.hashes.get(hash).add(image);
				} else {
					ArrayList<File> tmp = new ArrayList<File>();
					tmp.add(image);
					this.hashes.put(hash, tmp);
				}
				dProgress += 100/(double)iNumberOfImagesInFolders; 
				if ((int)dProgress > getProgress()) setProgress((int)dProgress);
			}
			this.progressMonitor.setProgress(100);
		}
		
		// reform
		// change the map to an ArrayList containing JSONArrays of 100 JSONObjects
		ArrayList<JSONArray> alJSONArrays = new ArrayList<JSONArray>();
		alJSONArrays.add(new JSONArray());
		//int actIdx = 0;
		Iterator<String> iter = this.hashes.keySet().iterator();
		while (iter.hasNext()) {
			JSONObject obj = new JSONObject();
			obj.put("hash", iter.next());
			alJSONArrays.get(alJSONArrays.size() - 1).add(obj);
			
			if (alJSONArrays.get(alJSONArrays.size() - 1).size() >= 100) {
				alJSONArrays.add(new JSONArray());
			}
		}
		
		// download
		this.progressMonitor = new ProgressMonitor(this.pnl,
				this.lib.getValue("backup_online.download.progress.textdl"),
	            "", 0, 100);
		this.progressMonitor.setProgress(0);
		setProgress(0);
		try {
			for (int i = 0; i < alJSONArrays.size(); ++i) {
				if (isCancelled()) {
					this.pnl.getFilePanel().updateList();
					this.pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					setProgress(100);
					return null;
				}
				JSONArray arr = alJSONArrays.get(i);
				//System.out.println(arr.toString());
				ArrayList<TaggedImage> imgList = getDataFromServer(arr);
				if (imgList == null) {
					// abbruch, irgendein fehler
					JOptionPane.showMessageDialog(this.pnl, this.lib.getValue("backup_online.download.error.generic.text"),
							this.lib.getValue("backup_online.download.error.generic.title"), JOptionPane.ERROR_MESSAGE);
					this.pnl.getFilePanel().updateList();
					this.pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return null;
				}
				
				// write data in db
				// Image lacks ID, path, width and height
				for (TaggedImage img : imgList) {
					img.changed = false;
					for (File f : this.hashes.get(img.hash)) {
						img.path = f;
						// backwards compatibility to version 1.0
						if (img.width == 0 || img.height == 0) {
							try {
								img.width = Jimi.getRasterImage(f.getAbsolutePath()).getWidth();
								img.height = Jimi.getRasterImage(f.getAbsolutePath()).getHeight();
							} catch (JimiException j) {
								System.err.println("Can't load image " + f.getAbsolutePath() + " to get resolution");
								img.width = img.height = 0;
							}
						}
						DbHandler.getInstance().addImageToDatabase(img);
						this.pnl.getFilePanel().setFileStateIndexedNoUpdate(f);
						++this.totalAmountOfImgs;
					}
				}
				setProgress((int)(100*((double)(i+1)/(double)alJSONArrays.size())));
			}
			this.progressMonitor.setProgress(100);
			// feddich
			JOptionPane.showMessageDialog(this.pnl, String.format(this.lib.getValue("backup_online.download.finish.text"), this.totalAmountOfImgs),
					this.lib.getValue("backup_online.upload.finish.title"), JOptionPane.INFORMATION_MESSAGE);
			this.pnl.getFilePanel().updateList();
			this.pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			return null;
		} catch (Exception e) {
			// fehler
			e.printStackTrace();
			JOptionPane.showMessageDialog(this.pnl, this.lib.getValue("backup_online.download.error.generic.text"),
					this.lib.getValue("backup_online.download.error.generic.title"), JOptionPane.ERROR_MESSAGE);
			this.pnl.getFilePanel().updateList();
			this.pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			this.progressMonitor.setProgress(100);
			return null;
		}
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if ("progress" == evt.getPropertyName() ) {
			int progress = (Integer) evt.getNewValue();
			this.progressMonitor.setProgress(progress);
            String message = String.format(this.lib.getValue("backup_online.download.progress.note"), progress);
            this.progressMonitor.setNote(message);
            if (this.progressMonitor.isCanceled() || isDone()) {
                Toolkit.getDefaultToolkit().beep();
                if (this.progressMonitor.isCanceled()) {
                    cancel(true);
                    System.gc();
                } else {
        			System.gc();
                }
                this.pnl.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            }
		}
	}
	
	private static ArrayList<File> getImagesFromFolder(File folder) {
		ArrayList<File> imagePaths = new ArrayList<File>();
		if (!folder.isDirectory()) return imagePaths;
		File[] files = folder.listFiles();
		if (files.length == 0) return imagePaths;
		for (File f : files) {
			String path = f.getAbsolutePath().toLowerCase();
			// only GIF, JPEG, TIFF, PNG, PICT, Photoshop, BMP, Targa, ICO, CUR,
			// Sunraster, XBM, XPM, and PCX. see com.sun.jimi.core.decoder.*
			if (path.endsWith(".apf") || path.endsWith(".bmp") ||
					path.endsWith(".cur") || path.endsWith(".gif") ||
					path.endsWith(".ico") || path.endsWith(".pcx") ||
					path.endsWith(".pict") || path.endsWith(".png") ||
					path.endsWith(".psd") || path.endsWith(".sunraster") ||
					path.endsWith(".tga") || path.endsWith(".tiff") ||
					path.endsWith(".xbm") || path.endsWith(".xpm") ||
					path.endsWith(".jpg") || path.endsWith(".jpeg") ||
					path.endsWith(".tif")) {
				imagePaths.add(f);
			}
		}
		return imagePaths;
	}
	
	private ArrayList<TaggedImage> getDataFromServer(JSONArray array) throws IOException {
		File f = saveFile(array, "wackydeli.txt");
		TreeMap<String, String> mParams = new TreeMap<String, String>();
		mParams.put("function", "download");
		mParams.put("param", f.getAbsolutePath());
		ArrayList<String> al = Downloader.sendFileUsingPOST(new URL(ServerCommunication.PHP_URL), mParams, f, null, "YiffIndex", 60000, 3);
		f.delete();
		if (al == null || al.get(0).equalsIgnoreCase("false")) return null;
		for (String s : al) System.out.println("getDataFromServer: " + s);
		
		
		JSONArray answerArray = null;
		try {
			answerArray = (JSONArray)JSONValue.parse(al.get(0));
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		if (answerArray == null) return null;
		ArrayList<TaggedImage> returnArray = new ArrayList<TaggedImage>();
		for (Object obj : answerArray) {
			TaggedImage img = new TaggedImage();
			img.fromJSONObject((JSONObject)obj);
			returnArray.add(img);
		}
		return returnArray;
	}
	
	private File saveFile(JSONArray array, String filename) throws IOException {
		File file = new File(DirHandler.getAppDataDir(), filename);
		BufferedWriter output = new BufferedWriter(new FileWriter(file));
		output.write(array.toString());
		output.close();
		return file;
	}
}
