package index;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import allrounder.IdString;
import allrounder.SortedListModel;
import allrounder.StringLib;
import allrounder.ToolTipList;

public class FetishChooserDialog extends JDialog implements ActionListener{
	private static final long serialVersionUID = -9170102919199424694L;

	StringLib lib = StringLib.getInstance();

	public static final int SAVE = 1;
	public static final int ABORT = 2;
	int buttonNumber = 0;
	
	IdString[] previousResultValue = null;
	
	SortedListModel fetishListMdl = new SortedListModel();
	SortedListModel resultListMdl = new SortedListModel();
	ToolTipList fetishList = new ToolTipList(this.fetishListMdl);
	ToolTipList resultList = new ToolTipList(this.resultListMdl);
	
	JLabel lblChosedFetishes = new JLabel(this.lib.getValue("fetishchooser.chosedfetishes.text"));
	
	JButton btnAdd, btnRemove, btnSave, btnAbort;
	
	TreeSet<IdString> fetishSet = new TreeSet<IdString>();
	
	public FetishChooserDialog() {
		setTitle(this.lib.getValue("fetishchooser.title.text"));
		setModal(true);

		setContentPane(new JPanel());
		((JPanel)getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		this.fetishList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.fetishList.addKeyListener(new KeyAdapter() {
			@Override
			public void keyTyped(KeyEvent e) {
				if(e.getKeyChar() == '\n') {
					FetishChooserDialog.this.btnAdd.doClick();
				} else {
					super.keyTyped(e);
				}
			}
		});
		this.fetishList.addMouseListener(new MouseAdapter() {
			 @Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					FetishChooserDialog.this.btnAdd.doClick();
				}
			}
		});
		
		this.resultList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.resultList.addKeyListener(new KeyAdapter() {
			@Override
			public void keyTyped(KeyEvent e) {
				if(e.getKeyChar() == '\n') {
					FetishChooserDialog.this.btnRemove.doClick();
				} else {
					super.keyTyped(e);
				}
			}
		});
		this.resultList.addMouseListener(new MouseAdapter() {
			 @Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					FetishChooserDialog.this.btnRemove.doClick();
				}
			}
		});
		
		// get fetishes
		int fetishno = 0;
		while(true) {
			String tempText = null;
			String tempToolTip = null;
			try {
				tempText = this.lib.getValue("fetishchooser.fetishes." + fetishno + ".text");
				tempToolTip = this.lib.getValue("fetishchooser.fetishes." + fetishno + ".tt");
			} catch (IllegalArgumentException e) {
				// this WILL be thrown because i just iterate as long as there is no entry left
			}
			if (tempText == null) break;
			if (tempToolTip != null && !tempToolTip.equals("")) { // tooltip is optional in the language file
				this.fetishSet.add(new IdString(fetishno, tempText, tempToolTip));
			} else {
				this.fetishSet.add(new IdString(fetishno, tempText));
			}
			++fetishno;
		}
		
		// fill the list
		for (IdString s : this.fetishSet) {
			this.fetishListMdl.addElement(s);
		}
		
		this.btnAdd = makeButton(this.lib.getValue("fetishchooser.add.text"), "ADD", this.lib.getValue("fetishchooser.add.tt"));
		this.btnRemove = makeButton(this.lib.getValue("fetishchooser.remove.text"), "REMOVE", this.lib.getValue("fetishchooser.remove.tt"));
		this.btnAdd.setFont(new Font("sansserif", Font.PLAIN, 10));
		this.btnRemove.setFont(new Font("sansserif", Font.PLAIN, 10));
		this.btnRemove.setEnabled(false);
		this.btnSave = makeButton(this.lib.getValue("fetishchooser.save.text"), "SAVE", this.lib.getValue("fetishchooser.save.tt"));
		this.btnAbort = makeButton(this.lib.getValue("fetishchooser.abort.text"), "ABORT", this.lib.getValue("fetishchooser.abort.tt"));
		
//		---> waagrecht
		double[][] layout = {{TableLayoutConstants.FILL, 10, this.btnAdd.getPreferredSize().width, 10, TableLayoutConstants.FILL},
							 // senkrecht
							 {this.lblChosedFetishes.getPreferredSize().height, 10, TableLayoutConstants.FILL, this.btnAdd.getPreferredSize().height, this.btnRemove.getPreferredSize().height, TableLayoutConstants.FILL, 10, this.btnSave.getPreferredSize().height, this.btnSave.getPreferredSize().height}};
		setLayout(new TableLayout(layout));
		
		add(new JScrollPane(this.fetishList), "0, 2, 0, 5");
		add(this.btnAdd, "2, 3");
		add(this.btnRemove, "2, 4");
		add(this.lblChosedFetishes, "4, 0");
		add(new JScrollPane(this.resultList), "4, 2, 4, 5");
		add(this.btnSave, "4, 7");
		add(this.btnAbort, "4, 8");
		
		setSize(getPreferredSize());
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				abort();
			}
		});
	}
	
	public void actionPerformed(ActionEvent arg0) {
		String command = arg0.getActionCommand();
		if(command.equalsIgnoreCase("ADD")) {
			Object selectedVal = this.fetishList.getSelectedValue();
			if (selectedVal == null) return;
			int idx = this.fetishList.getSelectedIndex();
			this.resultListMdl.addElement(selectedVal);
			this.btnRemove.setEnabled(true);
			this.fetishListMdl.removeElement(selectedVal);
			
			this.resultList.setSelectedIndex(this.resultListMdl.getIndexOfLastAdded());
			this.resultList.ensureIndexIsVisible(this.resultListMdl.getIndexOfLastAdded());
			
			if (this.fetishListMdl.size() == 0) {
				this.btnAdd.setEnabled(false);
				return;
			}
			if (this.fetishListMdl.size() - 1 < idx) {
				this.fetishList.setSelectedIndex(this.fetishListMdl.size() - 1);
			} else {
				this.fetishList.setSelectedIndex(idx);
			}
		} else if(command.equalsIgnoreCase("REMOVE")) {
			Object selectedVal = this.resultList.getSelectedValue();
			if (selectedVal == null) return;
			int idx = this.resultList.getSelectedIndex();
			this.resultListMdl.removeElement(selectedVal);
			this.btnAdd.setEnabled(true);
			this.fetishListMdl.addElement(selectedVal);
			
			this.fetishList.setSelectedIndex(this.fetishListMdl.getIndexOfLastAdded());
			this.fetishList.ensureIndexIsVisible(this.fetishListMdl.getIndexOfLastAdded());
			
			if (this.resultListMdl.size() == 0) {
				this.btnRemove.setEnabled(false);
				return;
			}
			if (this.resultListMdl.size() - 1 < idx) {
				this.resultList.setSelectedIndex(this.resultListMdl.size() - 1);
			} else {
				this.resultList.setSelectedIndex(idx);
			}
		} else if(command.equalsIgnoreCase("SAVE")) {
			this.buttonNumber = SAVE;
			setVisible(false);
		} else if(command.equalsIgnoreCase("ABORT")) {
			abort();
		}
	}
	
	public void abort() {
		this.buttonNumber = ABORT;
		this.resultListMdl.removeAllElements();
		for (int i = 0; i < this.previousResultValue.length; ++i) {
			this.resultListMdl.addElement(this.previousResultValue[i]);
		}
		
		this.fetishListMdl.removeAllElements();
		for (IdString s : this.fetishSet) {
			if (!this.resultListMdl.contains(s))	this.fetishListMdl.addElement(s);
		}
		setVisible(false);
	}
	
	public int displayFetishChooserDialog() {
		// save current result-list-values in case the user presses "abort"
		this.previousResultValue = new IdString[this.resultListMdl.getSize()];
		for (int i = 0; i < this.resultListMdl.getSize(); ++i) {
			this.previousResultValue[i] = (IdString)this.resultListMdl.get(i);
		}
		setVisible(true);
		return this.buttonNumber;
	}
	
	public TreeSet<Integer> getFetishIds() {
		TreeSet<Integer> fetishIds = new TreeSet<Integer>();
		for (int i = 0; i < this.resultListMdl.getSize(); ++i) {
			fetishIds.add(((IdString)this.resultListMdl.get(i)).getID());
		}
		return fetishIds;
	}
	
	public void setFetishIds(TreeSet<Integer> fetish) {
		ArrayList<IdString> tmpList = new ArrayList<IdString>();
		for (int i = 0; i < this.fetishListMdl.getSize(); ++i) {
			if(fetish.contains(((IdString)this.fetishListMdl.get(i)).getID())) {
				tmpList.add((IdString)this.fetishListMdl.get(i));
			}
		}
		
		for (IdString str : tmpList) {
			this.fetishListMdl.removeElement(str);
			this.resultListMdl.addElement(str);
		}
		
		if (this.fetishListMdl.size() == 0) {
			this.btnAdd.setEnabled(false);
		} else {
			this.btnAdd.setEnabled(true);
		}
		
		if (this.resultListMdl.size() == 0) {
			this.btnRemove.setEnabled(false);
		} else {
			this.btnRemove.setEnabled(true);
		}
	}
	
	private JButton makeButton(String text,	String actionCommand, String toolTipText) {
		// Create and initialize the button.
		JButton button = new JButton();
		button.setActionCommand(actionCommand);
		button.setToolTipText(toolTipText);
		button.addActionListener(this);
		button.setText(text);

		return button;
	}
}
