package index;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.swing.ButtonGroup;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionListener;

import allrounder.DbHandler;
import allrounder.StringLib;
import allrounder.TaggedImage;
import backup.local.Export;
import backup.local.Import;

public class FilePanel extends JPanel implements ActionListener {
	private static final long serialVersionUID = -6371315754543885511L;
	DefaultListModel listModel = new DefaultListModel();
	ColorListCellRenderer listRenderer = new ColorListCellRenderer();
	JList fileList = new JList();
	JRadioButton btnIndiz;
	JRadioButton btnNotIndiz;
	JRadioButton btnAll;
	ButtonGroup btnGroup = new ButtonGroup();
	JButton btnChooseFolders;
	StringLib lib = StringLib.getInstance();
	FolderChooserDialog folderDialog = new FolderChooserDialog();
	
	TreeSet<ListItem> FileSet = new TreeSet<ListItem>();
	
	public FilePanel() {
		this.btnIndiz = buildBtn(this.btnIndiz, "INDIZ", this.lib.getValue("index.FilePanel.indiz.text"), this.lib.getValue("index.FilePanel.indiz.tt"));
		this.btnNotIndiz = buildBtn(this.btnNotIndiz, "NOTINDIZ", this.lib.getValue("index.FilePanel.notindiz.text"), this.lib.getValue("index.FilePanel.notindiz.tt"));
		this.btnAll = buildBtn(this.btnAll, "ALL", this.lib.getValue("index.FilePanel.all.text"), this.lib.getValue("index.FilePanel.all.tt"));
		this.btnAll.setSelected(true);
		this.btnGroup.add(this.btnIndiz);
		this.btnGroup.add(this.btnAll);
		this.btnGroup.add(this.btnNotIndiz);
		this.btnChooseFolders = new JButton(this.lib.getValue("index.FilePanel.choosefolder.text"));
		this.btnChooseFolders.setActionCommand("CHOOSEFOLDER");
		this.btnChooseFolders.setToolTipText(this.lib.getValue("index.FilePanel.choosefolder.tt"));
		this.btnChooseFolders.addActionListener(this);
		
		this.fileList.setFont(new Font("sansserif", Font.PLAIN, 10));
		this.fileList.setModel(this.listModel);
		this.fileList.setCellRenderer(this.listRenderer);
		intialise();
		
		int tempHeight = this.btnNotIndiz.getPreferredSize().height;
		                  //row / column
		double[][] layout = {{this.btnNotIndiz.getPreferredSize().width, TableLayoutConstants.FILL},{TableLayoutConstants.FILL, tempHeight, tempHeight, tempHeight}};
		this.setLayout(new TableLayout(layout));
		this.add(new JScrollPane(this.fileList), "0, 0, 1, 0");
		this.add(this.btnAll, "0, 1");
		this.add(this.btnChooseFolders, "1, 1, 1, 2");
		this.add(this.btnNotIndiz, "0, 2");
		this.add(this.btnIndiz, "0, 3");
	}
	
	public void intialise() {
		TreeMap<File, Boolean> folders = DbHandler.getInstance().getDirectories();
		ArrayList<File> imagePaths = new ArrayList<File>();
		Iterator<File> folderIter = folders.keySet().iterator();
    	while (folderIter.hasNext()) {
    		File f = folderIter.next();
    		if (folders.get(f) && (!f.exists())) continue;
			imagePaths.addAll(getImagesFromFolder(f));
		}
		
		this.FileSet.clear();
		ArrayList<Boolean> inDBList = DbHandler.getInstance().areImagesInDb(imagePaths);
		for (int i = 0; i < imagePaths.size(); ++i) {
			this.FileSet.add(new ListItem(imagePaths.get(i), inDBList.get(i)));
		}
		buildList();
	}
	
	public void setFileStateIndexedNoUpdate(File f) {
		for (ListItem li : this.FileSet) {
			if (li.file.equals(f)) {
				li.b = true;
				break;
			}
		}
	}
	
	public void setFileStateNotIndexedNoUpdate(File f) {
		for (ListItem li : this.FileSet) {
			if (li.file.equals(f)) {
				li.b = false;
				break;
			}
		}
	}
	
	public void setFileStateIndexed(File f) {
		setFileStateIndexedNoUpdate(f);
		updateList();
	}
	
	public void setFileStateNotIndexed(File f) {
		setFileStateNotIndexedNoUpdate(f);
		updateList();
	}
	
	public void updateList() {
		ListItem selection = (ListItem)this.fileList.getSelectedValue();
		buildList();
		if (selection == null) return;
		if (this.listModel.indexOf(selection) == -1) return;
		this.fileList.setSelectedValue(selection, true);
	}
	
	public void buildList() {
		if (!this.btnAll.isSelected()) {
			if (this.btnIndiz.isSelected()) buildList(true);
			if (this.btnNotIndiz.isSelected()) buildList(false);
			return;
		}
		clearList();
		ArrayList<ListItem> tmp = new ArrayList<ListItem>();
		for (ListItem li : this.FileSet) {
			tmp.add(li);
		}
		addElementsToList(tmp);
	}
	
	public void buildList (boolean showIsIndexed) {
		clearList();
		ArrayList<ListItem> tmp = new ArrayList<ListItem>();
		for (ListItem li : this.FileSet) {
			if (li.b == showIsIndexed) tmp.add(li);
		}
		addElementsToList(tmp);
	}
	
	private void addElementsToList(ArrayList<ListItem> elements) {
		DefaultListModel mdl = new DefaultListModel();
		for (ListItem li : elements) {
			mdl.addElement(li);
			if (!li.b) {
				this.listRenderer.addColor(mdl.getSize() - 1, Color.RED);
			}
		}
		this.listModel = mdl;
		this.fileList.setModel(this.listModel);
	}
	
	private void clearList() {
		this.listModel.removeAllElements();
		this.listRenderer.resetColors();
	}
	
	private JRadioButton buildBtn(JRadioButton btn, String ActionCommand, String text, String tooltip) {
		btn = new JRadioButton(text);
		btn.setActionCommand(ActionCommand);
		btn.setToolTipText(tooltip);
		btn.addActionListener(this);
		btn.setFont(new Font("sansserif", Font.PLAIN, 10));
		return btn;
	}
	
	public void untoggleAll() {
		this.btnIndiz.setSelected(false);
		this.btnAll.setSelected(false);
		this.btnNotIndiz.setSelected(false);
	}
	
	public void actionPerformed(ActionEvent ae) {
		String s = ae.getActionCommand();
		if (s.equalsIgnoreCase("INDIZ")) {
			updateList();
		} else if (s.equalsIgnoreCase("NOTINDIZ")) {
			updateList();
		} else if (s.equalsIgnoreCase("ALL")) {
			updateList();
		} else if (s.equalsIgnoreCase("CHOOSEFOLDER")){
			int retVal = this.folderDialog.displayFolderChooserDialog();
			if (retVal != SpeziesChooserDialog.SAVE) return;
			TreeMap<File, Boolean> folders = this.folderDialog.getFolders();
			
			ArrayList<File> deletedFolders = this.folderDialog.getRemovedFolders();
			
			ArrayList<File> imagePaths = new ArrayList<File>();
			
			Iterator<File> iterFolders = folders.keySet().iterator();
			while (iterFolders.hasNext()) {
				File f = iterFolders.next();
				DbHandler.getInstance().addFolderToDatabase(f, folders.get(f));
				imagePaths.addAll(getImagesFromFolder(f));
			}
			
			if (deletedFolders.size() != 0) {
				//check if there were images in these folders
				ArrayList<TaggedImage> lostImages = new ArrayList<TaggedImage>();
				for (File f : deletedFolders) {
					lostImages.addAll(DbHandler.getInstance().getImagesOfDirectory(f.getAbsolutePath()));
				}

				if (lostImages.size() != 0) {
					// does the user want to rescue the data?
					int iResult = JOptionPane.showConfirmDialog(this.getParent(), this.lib.getValue("index.FilePanel.lostdirs.text", lostImages.size() + " ", ""), this.lib.getValue("index.FilePanel.lostdirs.title"), JOptionPane.YES_NO_OPTION);
					if (iResult == JOptionPane.YES_OPTION) {
						Import.searchFilesForHashes(lostImages, (IndexPanel)this.getParent(), deletedFolders, false);
						// because of how FUCKING STUPID java did the whole progressmonitor thing, I'm unable to
						// get the info if it was aborted or not because when I get here, the FUCKING springworker may still run
						// goddamnit. So the wannabe-deleted data will stay until the next reorg. freaking hell sometimes i hate java
						
						/*if (wasAborted) {
							// delete everything anyway, therefore change iResult to NO
							iResult = JOptionPane.NO_OPTION;
						}*/
					}
					if (iResult != JOptionPane.YES_OPTION) {
						//delete images
						int iResult2 = JOptionPane.showConfirmDialog(this.getParent(), this.lib.getValue("backup_local.Import.nonrecdel.text", lostImages.size() + " ", ""), this.lib.getValue("backup_local.Import.nonrecdel.title"), JOptionPane.YES_NO_OPTION);
						if (iResult2 == JOptionPane.YES_OPTION) {
							Export.doExport(lostImages, (IndexPanel)this.getParent());
						}
						for (TaggedImage img : lostImages) {
							System.out.println("Debug: Images to delete are:");
							System.out.println(img);
							if (img.path != null) ((IndexPanel)this.getParent()).getFilePanel().setFileStateNotIndexedNoUpdate(img.path);
							DbHandler.getInstance().removeImageFromDatabase(img.ID);
						}
					}
					lostImages = null;
				}
				
				for (File f : deletedFolders) {
					DbHandler.getInstance().removeFolderFromDatabase(f);
				}
			}
			this.getParent().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			generateFileSet(imagePaths);
			this.getParent().setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}
	
	public void generateFileSet(ArrayList<File> imagePaths) {
		this.FileSet.clear();
		for (File f : imagePaths) {
			this.FileSet.add(new ListItem(f, DbHandler.getInstance().isImageInDb(f)));
		}
		buildList();
	}
	
	public void changeColor(File f, boolean bIsIndexed) {
		for (ListItem li : this.FileSet) {
			if (li.file.equals(f)) {
				li.b = bIsIndexed;
				break;
			}
		}
	}
	
	public static ArrayList<File> getImagesFromFolder (File folder) {
		ArrayList<File> imagePaths = new ArrayList<File>();
		if (!folder.isDirectory()) return imagePaths;
		File[] files = folder.listFiles();
		if (files == null) {
			System.err.println("IOError on trying to read directory " + folder.getAbsolutePath());
			return imagePaths;
		}
		if (files.length == 0) return imagePaths;
		for (File f : files) {
			String path = f.getAbsolutePath().toLowerCase();
			// only GIF, JPEG, TIFF, PNG, PICT, Photoshop, BMP, Targa, ICO, CUR,
			// Sunraster, XBM, XPM, and PCX. see com.sun.jimi.core.decoder.*
			if (path.endsWith(".apf") || path.endsWith(".bmp") ||
					path.endsWith(".cur") || path.endsWith(".gif") ||
					path.endsWith(".ico") || path.endsWith(".pcx") ||
					path.endsWith(".pict") || path.endsWith(".png") ||
					path.endsWith(".psd") || path.endsWith(".sunraster") ||
					path.endsWith(".tga") || path.endsWith(".tiff") ||
					path.endsWith(".xbm") || path.endsWith(".xpm") ||
					path.endsWith(".jpg") || path.endsWith(".jpeg") ||
					path.endsWith(".tif")) {
				imagePaths.add(f);
			}
		}
		return imagePaths;
	}
	
	public void addListSelectionListener(ListSelectionListener al) {
		this.fileList.addListSelectionListener(al);
	}
	
	public File getImage() {
		if (this.fileList.isSelectionEmpty()) return null;
		ListItem li = (ListItem)this.fileList.getSelectedValue();
		return li.file;
	}
	
	public File getImageAt(int row) {
		if (row < 0 || row > this.fileList.getModel().getSize()) return null;
		ListItem li = (ListItem)this.fileList.getModel().getElementAt(row);
		return li.file;
	}
	public boolean hasNext() {
		if (this.fileList.getSelectedIndex() < this.listModel.getSize() - 1) return true;
		return false;
	}
	
	public boolean hasPrevious() {
		if (this.fileList.getSelectedIndex() >0) return true;
		return false;
	}
	
	public void next() {
		if (hasNext()) this.fileList.setSelectedIndex(this.fileList.getSelectedIndex() + 1);
	}
	
	public void previous() {
		if (hasPrevious()) this.fileList.setSelectedIndex(this.fileList.getSelectedIndex() - 1);
	}

	/**
	 * @return the fileSet
	 */
	public TreeSet<ListItem> getFileSet() {
		return FileSet;
	}

	/**
	 * @param fileSet the fileSet to set
	 */
	public void setFileSet(TreeSet<ListItem> fileSet) {
		FileSet = fileSet;
	}

	public class ListItem implements Comparable<ListItem> {
		File file;
		boolean b;
		
		public ListItem() {}
		public ListItem(File f, boolean b) {
			this.file = f;
			this.b = b;
		}
		
		public int compareTo(ListItem li) {
			return this.file.getAbsolutePath().compareToIgnoreCase(li.file.getAbsolutePath());
		}
		
		/* (non-Javadoc)
		 * @see java.lang.Object#hashCode()
		 */
		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + (this.b ? 1231 : 1237);
			result = prime * result + ((this.file == null) ? 0 : this.file.hashCode());
			return result;
		}
		/* (non-Javadoc)
		 * @see java.lang.Object#equals(java.lang.Object)
		 */
		@Override
		public boolean equals(Object obj) {
			if (this == obj) return true;
			if (obj == null) return false;
			if (getClass() != obj.getClass()) return false;
			ListItem other = (ListItem) obj;
			if (this.b != other.b) return false;
			if (this.file == null) {
				if (other.file != null) return false;
			} else if (!this.file.equals(other.file)) {
				return false;
			}
			return true;
		}
		
		@Override
		public String toString() {
			return this.file.getAbsolutePath();
		}
	}
}

class ColorListCellRenderer extends DefaultListCellRenderer {
	private static final long serialVersionUID = 9130118172924442594L;
	TreeMap<Integer, Color> coloridx = new TreeMap<Integer, Color>();
	
	@Override
	public Component getListCellRendererComponent(JList list, Object value,
			int index, boolean isSelected, boolean cellHasFocus) {
		JLabel label = (JLabel) super.getListCellRendererComponent(list, value,
				index, isSelected, cellHasFocus);

		Color c = this.coloridx.get(index);
		if (c != null) {
			label.setForeground(c);
		}
		
		return label;
	}
	
	public void addColor (int index, Color color) {
		this.coloridx.put(index, color);
	}
	
	public void resetColors() {
		this.coloridx.clear();
	}
} 
