package index;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.TreeMap;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import allrounder.DbHandler;
import allrounder.StringLib;

public class FolderChooserDialog extends JDialog implements ActionListener{
	private static final long serialVersionUID = -4984954535658675775L;

	StringLib lib = StringLib.getInstance();

	public static final int SAVE = 1;
	public static final int ABORT = 2;
	int buttonNumber = 0;
	
	ArrayList<File> addedDirs = new ArrayList<File>();
	ArrayList<File> removedDirs = new ArrayList<File>();
	
	TreeMap<File, Boolean> previousResultValue = null;
	
	FolderChooserTable tblResult;
	FolderChooserTableModel mdlResult;
	
	JButton btnAdd, btnRemove, btnSave, btnAbort;
	
	JFileChooser fileChooser = new JFileChooser();
	
	public FolderChooserDialog() {
		setTitle(this.lib.getValue("index.FolderChooserDialog.title.text"));
		setModal(true);

		setContentPane(new JPanel());
		((JPanel)getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		String[] columnNames = {this.lib.getValue("index.FolderChooserDialog.path.text"),
				this.lib.getValue("index.FolderChooserDialog.removMedia.text")};
		this.mdlResult = new FolderChooserTableModel(null, columnNames);
		this.tblResult = new FolderChooserTable(this.mdlResult);
		
		TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(); 
		this.tblResult.setRowSorter(sorter);
        sorter.setModel(this.mdlResult);
		
		// damit der header auch zweispaltig sein kann
		TableCellRenderer renderer = this.tblResult.getTableHeader().getDefaultRenderer();
	    TableColumnModel model = this.tblResult.getColumnModel();
	    for (int j = 0; j < model.getColumnCount(); j++) {
	    	model.getColumn(j).setHeaderRenderer(renderer);
	    	if (j==1) {
	    		model.getColumn(j).setMinWidth(new JCheckBox().getPreferredSize().width);
	    		model.getColumn(j).setMaxWidth(new JLabel(this.lib.getValue("index.FolderChooserDialog.removMedia.text")).getPreferredSize().width + 5);
	    		model.getColumn(j).setPreferredWidth(100); // 100%, also maxwidth
	    	}
	    }
		
		this.btnAdd = makeButton(this.lib.getValue("index.FolderChooserDialog.add.text"), "ADD", null);
		this.btnRemove = makeButton(this.lib.getValue("index.FolderChooserDialog.remove.text"), "REMOVE", null);
		this.btnRemove.setEnabled(false);
		this.btnSave = makeButton(this.lib.getValue("index.FolderChooserDialog.save.text"), "SAVE", this.lib.getValue("index.FolderChooserDialog.save.tt"));
		this.btnAbort = makeButton(this.lib.getValue("index.FolderChooserDialog.abort.text"), "ABORT", this.lib.getValue("index.FolderChooserDialog.abort.tt"));
		
		int btnwidth = (this.btnAdd.getPreferredSize().width > this.btnRemove.getPreferredSize().width) ? this.btnAdd.getPreferredSize().width : this.btnRemove.getPreferredSize().width;
							// waagrecht
		double[][] layout = {{btnwidth, TableLayoutConstants.FILL, this.btnAbort.getPreferredSize().width},
							 // senkrecht
							 {this.btnAdd.getPreferredSize().height, this.btnAdd.getPreferredSize().height, 10, TableLayoutConstants.FILL, 10, this.btnSave.getPreferredSize().height, this.btnSave.getPreferredSize().height}};
		setLayout(new TableLayout(layout));
		
		add(this.btnAdd, "0, 0");
		add(this.btnRemove, "0, 1");
		add(new JScrollPane(this.tblResult), "0, 3, 2, 3");
		add(this.btnSave, "2, 5");
		add(this.btnAbort, "2, 6");
		
		setSize(getPreferredSize());
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				abort();
			}
		});
		
		TreeMap<File, Boolean> dirs = DbHandler.getInstance().getDirectories();
		Iterator<File> iter = dirs.keySet().iterator();
		while (iter.hasNext()) {
			File f = iter.next();
			this.mdlResult.addRow(new Object[] {f.getAbsolutePath(), dirs.get(f)});
		}
		if (this.mdlResult.getRowCount() == 0) {
			this.btnRemove.setEnabled(false);
		} else {
			this.btnRemove.setEnabled(true);
			this.tblResult.getSelectionModel().setSelectionInterval(0, 0);
		}
		
		this.fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		this.fileChooser.setMultiSelectionEnabled(true);
		
		setDefaultCloseOperation(DISPOSE_ON_CLOSE); // the windowlistener above should probably handle that *shrug*
	}
	
	public void actionPerformed(ActionEvent arg0) {
		String command = arg0.getActionCommand();
		if(command.equalsIgnoreCase("ADD")) {
			int returnCode = this.fileChooser.showOpenDialog(this);
			if (returnCode != JFileChooser.APPROVE_OPTION) {
				return;
			}
			
			for (File f : this.fileChooser.getSelectedFiles()) {
				if (!f.isDirectory()) continue;
				if (!f.exists()) continue;
				File[] subFiles = f.listFiles();
				boolean bSubDirs = false;
				for (File sub : subFiles) {
					if (sub.isDirectory()) {
						bSubDirs = true;
						break;
					}
				}
				
				if (bSubDirs) {
					int answer = JOptionPane.showConfirmDialog(this, f.getName() + ": " + this.lib.getValue("index.FolderChooserDialog.subfolders.text"), this.lib.getValue("index.FolderChooserDialog.subfolders.title"), JOptionPane.YES_NO_OPTION);
					if (answer == JOptionPane.YES_OPTION) {
						this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
						ArrayList<File> subDirs = getAllSubFolders(f);
						for (File dir : subDirs) {
							this.mdlResult.addRow(new Object[] {dir.getAbsolutePath(), new Boolean(false)});
							if (!this.removedDirs.contains(dir)) {
								this.addedDirs.add(dir);
							} else {
								this.removedDirs.remove(dir);
							}
						}
						this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
						continue;
					}
				}
				this.mdlResult.addRow(new Object[] {f.getAbsolutePath(), new Boolean(false)});
				if (!this.removedDirs.contains(f)) {
					this.addedDirs.add(f);
				} else {
					this.removedDirs.remove(f);
				}
			}
			this.btnRemove.setEnabled(true);
		} else if(command.equalsIgnoreCase("REMOVE")) {
			int[] rowsToRemove = this.tblResult.getSelectedRows();
			if (rowsToRemove.length == 0) return;
			
			for (int i = 0; i < rowsToRemove.length; ++i) {
				rowsToRemove[i] = this.tblResult.convertRowIndexToModel(rowsToRemove[i]);
			}
			
			Arrays.sort(rowsToRemove);
			for (int i = rowsToRemove.length - 1; i >= 0; --i) {
				int rowIdx = rowsToRemove[i];
				String sPath = this.mdlResult.getValueAt(rowIdx,	0).toString();
				if (!this.addedDirs.contains(new File(sPath))) {
					this.removedDirs.add(new File(sPath));
				} else {
					this.addedDirs.remove(new File(sPath));
				}
				this.mdlResult.removeRow(rowIdx);
			}
			
			if (this.mdlResult.getRowCount() == 0) {
				this.btnRemove.setEnabled(false);
				return;
			}
			
			if (rowsToRemove.length != 1) {
				this.tblResult.getSelectionModel().setSelectionInterval(0, 0);
				return;
			}
			
			if (this.mdlResult.getRowCount() - 1 < rowsToRemove[0]) {
				this.tblResult.getSelectionModel().setSelectionInterval(this.mdlResult.getRowCount() - 1, this.mdlResult.getRowCount() - 1);
			} else {
				this.tblResult.getSelectionModel().setSelectionInterval(rowsToRemove[0], rowsToRemove[0]);
			}
		} else if(command.equalsIgnoreCase("SAVE")) {
			this.buttonNumber = SAVE;
			setVisible(false);
		} else if(command.equalsIgnoreCase("ABORT")) {
			abort();
		}
	}
	
	private ArrayList<File> getAllSubFolders (File input) {
		ArrayList<File> subFolders = new ArrayList<File>();
		if (!input.isDirectory()) return subFolders;
		subFolders.add(input);
		File[] subFiles = input.listFiles();
		if (subFiles == null || subFiles.length == 0) return subFolders;
		for (File f : subFiles) {
			subFolders.addAll(getAllSubFolders(f));
		}
		return subFolders;
	}
	
	public void abort() {
		this.buttonNumber = ABORT;
		setFolders(this.previousResultValue);
		setVisible(false);
		dispose();
	}
	
	public int displayFolderChooserDialog() {
		this.addedDirs.clear();
		this.removedDirs.clear();
		// save current result-list-values in case the user presses "abort"
		this.previousResultValue = getFolders();
		setVisible(true);
		return this.buttonNumber;
	}
	
	public TreeMap<File, Boolean> getFolders() {
		TreeMap<File, Boolean> folders = new TreeMap<File, Boolean>();
		for (int i = 0; i < this.mdlResult.getRowCount(); ++i) {
			folders.put(new File((String)this.mdlResult.getValueAt(i, 0)), ((Boolean)this.mdlResult.getValueAt(i, 1)));
		}
		return folders;
	}
	
	public ArrayList<File> getAddedFolders() {
		return this.addedDirs;
	}
	
	public ArrayList<File> getRemovedFolders() {
		return this.removedDirs;
	}
	
	public void setFolders(TreeMap<File, Boolean> folders) {
		this.mdlResult.removeAllRows();
		
		Iterator<File> iter = folders.keySet().iterator();
		while(iter.hasNext()) {
			File f = iter.next();
			this.mdlResult.addRow(new Object[] {f.getAbsolutePath(), folders.get(f)});
		}
		
		if (this.mdlResult.getRowCount() == 0) {
			this.btnRemove.setEnabled(false);
		} else {
			this.btnRemove.setEnabled(true);
			this.tblResult.getSelectionModel().setSelectionInterval(0, 0);
		}
	}
	
	private JButton makeButton(String text,	String actionCommand, String toolTipText) {
		// Create and initialize the button.
		JButton button = new JButton();
		button.setActionCommand(actionCommand);
		if (toolTipText != null) button.setToolTipText(toolTipText);
		button.addActionListener(this);
		button.setText(text);

		return button;
	}
}

class FolderChooserTable extends JTable {
	private static final long serialVersionUID = 3776873976550743304L;

	// whole class just to get tooltips to the checkboxes.. ugly as hell but that's java
	public FolderChooserTable(TableModel dm) {
		super(dm);
	}
	
	@Override
	public TableCellRenderer getCellRenderer(int row, int column) {
        return new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 3048388384248989066L;

			@Override
        	public Component getTableCellRendererComponent(JTable table,
        			Object value, boolean isSelected, boolean hasFocus, int row,
        			int column) {
        		if (!(value instanceof Boolean)) return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        		JCheckBox chk = new JCheckBox();
        		chk.setSelected((Boolean)value);
        		chk.setHorizontalAlignment(SwingConstants.CENTER);
        		chk.setToolTipText(StringLib.getInstance().getValue("index.FolderChooserDialog.removMedia.tt"));
        		if (isSelected) {
        			chk.setBackground(table.getSelectionBackground());
        		} else {
        			chk.setBackground(table.getBackground());
        		}
        		return chk;
        	}
        };
    }
}

class FolderChooserTableModel extends DefaultTableModel {
	private static final long serialVersionUID = 8139614334068586561L;

	public FolderChooserTableModel(Object[][] data, Object[] columnNames) {
		super(data, columnNames);
	}
	
	@Override
	public void addRow(Object[] row) {
		String rowSortString = row[0].toString();
		for (int i = 0; i < getRowCount(); ++i) {
			int res = getValueAt(i, 0).toString().compareTo(rowSortString);
			if (res == 0) {
				// folder is allready in table
				return;
			}
		}
		super.addRow(row);
	}
	
	@Override
	public Class<?> getColumnClass(int columnIndex) {
		// defaulttablemodel would return object (not bool, therefore no checkbox)
		if (columnIndex >= getColumnCount() || getRowCount() == 0) return Object.class;
		return getValueAt(0, columnIndex).getClass();
	}

	public void removeAllRows() {
		for (int i = 0; i < getRowCount(); ++i) {
			removeRow(0); // after first row(0) was removed, row(1) becomes row(0)
		}
	}
	
	@Override
	public boolean isCellEditable(int row, int col) {
		if (getValueAt(row, col).getClass().equals(Boolean.class)) return true;
		return false;
	}
}