package index;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import allrounder.IdString;
import allrounder.SortedListModel;
import allrounder.StringLib;
import allrounder.ToolTipList;

public class SpeziesChooserDialog extends JDialog implements ActionListener {
	private static final long serialVersionUID = 7920739704973625712L;

	StringLib lib = StringLib.getInstance();

	public static final int SAVE = 1;
	public static final int ABORT = 2;
	int buttonNumber = 0;
	
	IdString[] previousResultValue = null;
	
	DefaultComboBoxModel familyDropDownMdl = new DefaultComboBoxModel();
	SortedListModel speziesListMdl = new SortedListModel();
	SortedListModel resultListMdl = new SortedListModel();
	JComboBox familyDropDown = new JComboBox(this.familyDropDownMdl);
	ToolTipList speziesList = new ToolTipList(this.speziesListMdl);
	ToolTipList resultList = new ToolTipList(this.resultListMdl);
	
	JLabel lblFilter = new JLabel(this.lib.getValue("spezieschooser.filter.text"));
	JLabel lblChosedSpezies = new JLabel(this.lib.getValue("specieschooser.chosedspecies.text"));
	
	JButton btnAdd, btnRemove, btnSave, btnAbort;
	
	//      family            spezies
	TreeMap<IdString, TreeSet<IdString>> speziesMap = new TreeMap<IdString, TreeSet<IdString>>();
	
	public SpeziesChooserDialog() {
		setTitle(this.lib.getValue("spezieschooser.title.text"));
		setModal(true);
		//setLocationRelativeTo(null);

		setContentPane(new JPanel());
		((JPanel)getContentPane()).setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		
		this.familyDropDown.setActionCommand("DROPDOWN");
		this.familyDropDown.addActionListener(this);
		this.speziesList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.speziesList.addKeyListener(new KeyAdapter() {
			@Override
			public void keyTyped(KeyEvent e) {
				if(e.getKeyChar() == '\n') {
					SpeziesChooserDialog.this.btnAdd.doClick();
				} else {
					super.keyTyped(e);
				}
			}
		});
		this.speziesList.addMouseListener(new MouseAdapter() {
			 @Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					SpeziesChooserDialog.this.btnAdd.doClick();
				}
			}
		});
		
		
		this.resultList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.resultList.addKeyListener(new KeyAdapter() {
			@Override
			public void keyTyped(KeyEvent e) {
				if(e.getKeyChar() == '\n') {
					SpeziesChooserDialog.this.btnRemove.doClick();
				} else {
					super.keyTyped(e);
				}
			}
		});
		this.resultList.addMouseListener(new MouseAdapter() {
			 @Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					SpeziesChooserDialog.this.btnRemove.doClick();
				}
			}
		});
		
		// get spezies/families
		int i = 0;
		while(true) {
			String temp = null;
			try {
				temp = this.lib.getValue(i + "");
			} catch (IllegalArgumentException e) {
				// this WILL be thrown because i just iterate as long as there is no entry left
				break;
			}
			String[] speziesArr = temp.split(",");
			TreeSet<IdString> speziSet = new TreeSet<IdString>();
			for (String s : speziesArr) {
				IdString spezies = new IdString(Integer.parseInt(s), this.lib.getValue("species." + s));
				// try to get tooltip (for modifiers)
				try {
					String tmp = this.lib.getValue("species." + s + ".tt");
					spezies.setToolTipText(tmp);
				} catch (IllegalArgumentException e) {
					// this WILL be thrown for any species without tooltip (most of em)
				}
				speziSet.add(spezies);
			}
			IdString family = new IdString(i, this.lib.getValue("family." + i));
			this.speziesMap.put(family, speziSet);
			++i;
		}
		
		// add possible families to the drop down box
		Iterator<IdString> iter = this.speziesMap.keySet().iterator();
		while (iter.hasNext()) {
			this.familyDropDownMdl.addElement(iter.next());
		}
		
		this.btnAdd = makeButton(this.lib.getValue("spezieschooser.add.text"), "ADD", this.lib.getValue("spezieschooser.add.tt"));
		this.btnRemove = makeButton(this.lib.getValue("spezieschooser.remove.text"), "REMOVE", this.lib.getValue("spezieschooser.remove.tt"));
		this.btnAdd.setFont(new Font("sansserif", Font.PLAIN, 10));
		this.btnRemove.setFont(new Font("sansserif", Font.PLAIN, 10));
		this.btnRemove.setEnabled(false);
		this.btnSave = makeButton(this.lib.getValue("spezieschooser.save.text"), "SAVE", this.lib.getValue("spezieschooser.save.tt"));
		this.btnAbort = makeButton(this.lib.getValue("spezieschooser.abort.text"), "ABORT", this.lib.getValue("spezieschooser.abort.tt"));
		
		//					---> waagrecht
		double[][] layout = {{this.lblFilter.getPreferredSize().width, TableLayoutConstants.FILL, 10, this.btnAdd.getPreferredSize().width, 10, TableLayoutConstants.FILL},
							 // senkrecht
							 {this.lblFilter.getPreferredSize().height, 10, TableLayoutConstants.FILL, this.btnAdd.getPreferredSize().height, this.btnRemove.getPreferredSize().height, TableLayoutConstants.FILL, 10, this.btnSave.getPreferredSize().height, this.btnSave.getPreferredSize().height}};
		setLayout(new TableLayout(layout));
		
		add(this.lblFilter, "0, 0");
		add(this.familyDropDown, "1, 0");
		add(new JScrollPane(this.speziesList), "0, 2, 1, 5");
		add(this.btnAdd, "3, 3");
		add(this.btnRemove, "3, 4");
		add(this.lblChosedSpezies, "5, 0");
		add(new JScrollPane(this.resultList), "5, 2, 5, 5");
		add(this.btnSave, "5, 7");
		add(this.btnAbort, "5, 8");
		
		setSize(getPreferredSize());
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				abort();
			}
		});
	}
	
	public void actionPerformed(ActionEvent arg0) {
		String command = arg0.getActionCommand();
		if (command.equalsIgnoreCase("DROPDOWN")) {
			this.speziesListMdl.removeAllElements();
			Iterator<IdString> iter = this.speziesMap.get(this.familyDropDownMdl.getSelectedItem()).iterator();
			while (iter.hasNext()) {
				IdString species = iter.next();
				if (!this.resultListMdl.contains(species)) {
					this.speziesListMdl.addElement(species);
				}
			}
		} else if(command.equalsIgnoreCase("ADD")) {
			Object selectedVal = this.speziesList.getSelectedValue();
			if (selectedVal == null) return;
			int idx = this.speziesList.getSelectedIndex();
			this.resultListMdl.addElement(selectedVal);
			this.btnRemove.setEnabled(true);
			this.speziesListMdl.removeElement(selectedVal);
			
			this.resultList.setSelectedIndex(this.resultListMdl.getIndexOfLastAdded());
			this.resultList.ensureIndexIsVisible(this.resultListMdl.getIndexOfLastAdded());
			
			if (this.speziesListMdl.size() == 0) {
				this.btnAdd.setEnabled(false);
				return;
			}
			if (this.speziesListMdl.size() - 1 < idx) {
				this.speziesList.setSelectedIndex(this.resultListMdl.size() - 1);
			} else {
				this.speziesList.setSelectedIndex(idx);
			}
		} else if(command.equalsIgnoreCase("REMOVE")) {
			Object selectedVal = this.resultList.getSelectedValue();
			if (selectedVal == null) return;
			int idx = this.resultList.getSelectedIndex();
			this.resultListMdl.removeElement(selectedVal);
			this.btnAdd.setEnabled(true);
			this.speziesListMdl.addElement(selectedVal);
			
			this.speziesList.setSelectedIndex(this.speziesListMdl.getIndexOfLastAdded());
			this.speziesList.ensureIndexIsVisible(this.speziesListMdl.getIndexOfLastAdded());
			
			if (this.resultListMdl.size() == 0) {
				this.btnRemove.setEnabled(false);
				return;
			}
			if (this.resultListMdl.size() - 1 < idx) {
				this.resultList.setSelectedIndex(this.resultListMdl.size() - 1);
			} else {
				this.resultList.setSelectedIndex(idx);
			}
		} else if(command.equalsIgnoreCase("SAVE")) {
			this.buttonNumber = SAVE;
			setVisible(false);
		} else if(command.equalsIgnoreCase("ABORT")) {
			abort();
		}
	}
	
	public void abort() {
		this.buttonNumber = ABORT;
		this.resultListMdl.removeAllElements();
		for (int i = 0; i < this.previousResultValue.length; ++i) {
			this.resultListMdl.addElement(this.previousResultValue[i]);
		}
		
		this.speziesListMdl.removeAllElements();
		Iterator<IdString> iter = this.speziesMap.get(this.familyDropDownMdl.getSelectedItem()).iterator();
		while (iter.hasNext()) {
			IdString species = iter.next();
			if (!this.resultListMdl.contains(species)) {
				this.speziesListMdl.addElement(species);
			}
		}
		setVisible(false);
	}
	
	public int displaySpeziesChooserDialog() {
		// save current result-list-values in case the user presses "abort"
		this.previousResultValue = new IdString[this.resultListMdl.getSize()];
		for (int i = 0; i < this.resultListMdl.getSize(); ++i) {
			this.previousResultValue[i] = (IdString)this.resultListMdl.get(i);
		}
		setVisible(true);
		return this.buttonNumber;
	}
	
	public TreeSet<Integer> getSpeciesIds() {
		TreeSet<Integer> speciesIds = new TreeSet<Integer>();
		for (int i = 0; i < this.resultListMdl.getSize(); ++i) {
			speciesIds.add(((IdString)this.resultListMdl.get(i)).getID());
		}
		return speciesIds;
	}
	
	public void setSpeciesIds(TreeSet<Integer> spezies) {
		ArrayList<IdString> tmpList = new ArrayList<IdString>();
		for (int i = 0; i < this.speziesListMdl.getSize(); ++i) {
			if(spezies.contains(((IdString)this.speziesListMdl.get(i)).getID())) {
				tmpList.add((IdString)this.speziesListMdl.get(i));
			}
		}
		
		for (IdString str : tmpList) {
			this.speziesListMdl.removeElement(str);
			this.resultListMdl.addElement(str);
		}
		
		if (this.speziesListMdl.size() == 0) {
			this.btnAdd.setEnabled(false);
		} else {
			this.btnAdd.setEnabled(true);
		}
		
		if (this.resultListMdl.size() == 0) {
			this.btnRemove.setEnabled(false);
		} else {
			this.btnRemove.setEnabled(true);
		}
	}
	
	private JButton makeButton(String text,	String actionCommand, String toolTipText) {
		// Create and initialize the button.
		JButton button = new JButton();
		button.setActionCommand(actionCommand);
		button.setToolTipText(toolTipText);
		button.addActionListener(this);
		button.setText(text);

		return button;
	}
}
