package main;

import index.IndexPanel;
import info.InfoDialog;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JToolBar;
import javax.swing.ToolTipManager;

import klaue.DownloadHelper.Downloader;
import search.SearchDialog;
import allrounder.DbHandler;
import allrounder.DirHandler;
import allrounder.StringLib;
import allrounder.TaggedImage;
import allrounder.logger.Logwriter;
import allrounder.md5.Hasher;
import allrounder.plugin.PluginBrowser;
import allrounder.plugin.PluginLoader;
import allrounder.plugin.YiffIndexPlugin;
import backup.local.Export;
import backup.local.Import;
import backup.online.ServerCommunication;


public class MainFrame extends JFrame implements ActionListener {
	private static final long serialVersionUID = -8080482500245483049L;
	IndexPanel m_indexPanel;
	JToolBar toolBar = new JToolBar();
	
	public MainFrame() {
		Downloader.installCookieManager();
		
		// set native md5 library paths
		String os_name = System.getProperty("os.name");
		String os_arch = System.getProperty("os.arch");
		if (os_name != null && os_arch != null) {
			String sNativeLibPath = null;
			os_name = os_name.toLowerCase();
			os_arch = os_arch.toLowerCase();
			if (os_name.equals("linux") &&
					(os_arch.equals("x86") ||
					 os_arch.equals("i386") ||
					 os_arch.equals("i486") ||
					 os_arch.equals("i586") ||
					 os_arch.equals("i686"))) {
				sNativeLibPath = new File(DirHandler.getAppPath() + File.separator + "allrounder" + File.separator + "md5" + File.separator +
						"linux_x86" + File.separator + "MD5.so").getAbsolutePath();
			} else if (os_name.startsWith("windows ") &&
					(os_arch.equals("x86") ||
					 os_arch.equals("i386") ||
					 os_arch.equals("i486") ||
					 os_arch.equals("i586") ||
					 os_arch.equals("i686"))) {
				sNativeLibPath = new File(DirHandler.getAppPath() + File.separator + "allrounder" + File.separator + "md5" + File.separator +
						"win32_x86" + File.separator + "MD5.dll").getAbsolutePath();
			} else if (os_name.startsWith("mac os x")) {
				if (os_arch.equals("ppc")) {
					sNativeLibPath = new File(DirHandler.getAppPath() + File.separator + "allrounder" + File.separator + "md5" + File.separator +
							"darwin_ppc" + File.separator + "MD5.jnilib").getAbsolutePath();
				} else if (os_arch.equals("x86") || os_arch.equals("i386")
						|| os_arch.equals("i486") || os_arch.equals("i586") || os_arch.equals("i686")) {
					sNativeLibPath = new File(DirHandler.getAppPath() + File.separator + "allrounder" + File.separator + "md5" + File.separator +
							"darwin_x86" + File.separator + "MD5.jnilib").getAbsolutePath();
				}
			}
			if (sNativeLibPath != null) System.setProperty("com.twmacinta.util.MD5.NATIVE_LIB_FILE", sNativeLibPath);
		}
		
		setTitle("YiffIndex");
		this.m_indexPanel = new IndexPanel();
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLayout(new BorderLayout());
		StringLib lib = StringLib.getInstance();
		this.toolBar.add(makeNavigationButton("general/Find24", "SEARCH", lib.getValue("MainFrame.search.tt"), lib.getValue("MainFrame.search.alt")));
		this.toolBar.add(makeNavigationButton("general/Import24", "EXPORT", lib.getValue("MainFrame.export.tt"), lib.getValue("MainFrame.export.alt")));
		this.toolBar.add(makeNavigationButton("general/Export24", "IMPORT", lib.getValue("MainFrame.import.tt"), lib.getValue("MainFrame.import.alt")));
		this.toolBar.add(makeNavigationButton("general/Upload", "UPLOAD", lib.getValue("MainFrame.upload.tt"), lib.getValue("MainFrame.upload.alt")));
		this.toolBar.add(makeNavigationButton("general/Download", "DOWNLOAD", lib.getValue("MainFrame.download.tt"), lib.getValue("MainFrame.download.alt")));
		this.toolBar.add(makePluginPopupButton("general/Preferences24", "PLUGIN", lib.getValue("MainFrame.plugin.tt"), lib.getValue("MainFrame.plugin.alt")));
		this.toolBar.add(new JSeparator());
		this.toolBar.add(makeNavigationButton("general/Refresh24", "DBREORG", lib.getValue("MainFrame.dbreorg.tt"), lib.getValue("MainFrame.dbreorg.alt")));
		this.toolBar.add(makeNavigationButton("general/Help24", "INFO", lib.getValue("MainFrame.info.tt"), lib.getValue("MainFrame.info.alt")));
		
		setSize(new Dimension(966, 700));
		setLocationRelativeTo(null);
        add(this.toolBar, BorderLayout.PAGE_START);
        add(this.m_indexPanel, BorderLayout.CENTER);
        ToolTipManager.sharedInstance().setDismissDelay(Integer.MAX_VALUE);
        ToolTipManager.sharedInstance().setInitialDelay(500);
        
        // catch shutdown (exit) to close database
    	Runtime.getRuntime().addShutdownHook(new Thread() {
    	    @Override
			public void run() { DbHandler.getInstance().destruct(); }
    	});
    	
    	//ImageIcon image = new ImageIco("img/icon.GIF");
    	this.setIconImage(Toolkit.getDefaultToolkit().createImage("icon.gif"));
    	
    	setVisible(true);
    	
    	if (Boolean.parseBoolean(DbHandler.getInstance().getGeneral("Firststart"))) {
    		DbHandler.getInstance().setGeneral("Firststart", "FALSE");
    		JOptionPane.showMessageDialog(this, lib.getValue("Firststart.text"), lib.getValue("Firststart.title"), JOptionPane.INFORMATION_MESSAGE);
    	}
	}
	
	protected JButton makeNavigationButton(String imageName,
			String actionCommand, String toolTipText, String altText) {
		// Look for the image.
		String imgLocation = "/toolbarButtonGraphics/" + imageName + ".gif";
		URL imageURL = getClass().getResource(imgLocation);

		// Create and initialize the button.
		JButton button = new JButton();
		button.setActionCommand(actionCommand);
		button.setToolTipText(toolTipText);
		button.addActionListener(this);

		if (imageURL != null) { // image found
			button.setIcon(new ImageIcon(imageURL, altText));
		} else { // no image found
			button.setText(altText);
			Logwriter.getInstance().writeLine("Resource not found: " + imgLocation, true);
		}

		return button;
	}
	
	protected JButton makePluginPopupButton(String imageName,
			String actionCommand, String toolTipText, String altText) {
		// Look for the image.
		String imgLocation = "/toolbarButtonGraphics/" + imageName + ".gif";
		URL imageURL = getClass().getResource(imgLocation);

		// Create and initialize the button.
		JButton button = new JButton();
		// button.setActionCommand(actionCommand);
		button.setToolTipText(toolTipText);
		button.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent e) {
				// Build Menu
				JPopupMenu pluginMenu = new JPopupMenu();
				
				// menu for all loaded plugins
				TreeMap<String, YiffIndexPlugin> tmPlugins = PluginLoader.getInstance().getLoadedPlugins();
				Iterator<String> iter = tmPlugins.keySet().iterator();
				while (iter.hasNext()) {
					String sClassName = iter.next();
					YiffIndexPlugin plugin = tmPlugins.get(sClassName);
					JMenuItem menuItem = new JMenuItem(plugin.getName());
					menuItem.setActionCommand(sClassName);
					menuItem.addActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent arg0) {
							YiffIndexPlugin plugin = PluginLoader.getInstance().getLoadedPlugins().get(arg0.getActionCommand());
							if (plugin != null) plugin.pluginCalled(MainFrame.this);
						}
					});
					menuItem.setFont(menuItem.getFont().deriveFont(menuItem.getFont().getStyle() ^ Font.BOLD));
					pluginMenu.add(menuItem);
				}
				
				pluginMenu.add(new JPopupMenu.Separator());
				
				// menu for PluginBrowser
				JMenuItem menuItem = new JMenuItem(StringLib.getInstance().getValue("MainFrame.plugin.browse"));
				menuItem.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
						new PluginBrowser(MainFrame.this);
					}
				});
				//menuItem.setFont(menuItem.getFont().deriveFont(menuItem.getFont().getStyle() ^ Font.BOLD));
				pluginMenu.add(menuItem);
				
				pluginMenu.show(e.getComponent(), 0, e.getComponent().getHeight());
			}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) {}
			
		});

		if (imageURL != null) { // image found
			button.setIcon(new ImageIcon(imageURL, altText));
		} else { // no image found
			button.setText(altText);
			Logwriter.getInstance().writeLine("Resource not found: " + imgLocation, true);
		}
		
		button.setActionCommand(actionCommand);

		return button;
	}

	
	public static void main(String[] args) {
		// redirect all unchecked exceptions to own exceptionhandler to write it in the log
		System.setProperty("sun.awt.exception.handler", "allrounder.logger.TopLevelExceptionHandler");
		new MainFrame();
	}

	public void actionPerformed(ActionEvent ae) {
		String sAcComm = ae.getActionCommand();
		if (sAcComm.equalsIgnoreCase("SEARCH")) {
			new SearchDialog(this.m_indexPanel.getFilePanel());
			System.out.println("srch");
		} else if (sAcComm.equalsIgnoreCase("EXPORT")) {
			Export.doExport(null, this.m_indexPanel);
			System.out.println("ex");
		} else if (sAcComm.equalsIgnoreCase("IMPORT")) {
			Import.doImport(this.m_indexPanel);
			System.out.println("imp");
		} else if (sAcComm.equalsIgnoreCase("UPLOAD")) {
			// ask user if he want to upload only one or all images
			StringLib lib = StringLib.getInstance();
			 Object[] possibleValues = {lib.getValue("MainFrame.upload.message.actImg"),
					 lib.getValue("MainFrame.upload.message.allImg"), lib.getValue("MainFrame.upload.message.cancel")};
			 int selectedValue = JOptionPane.showOptionDialog(this,
			             lib.getValue("MainFrame.upload.message.text"), lib.getValue("MainFrame.upload.message.title"),
			             JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null,
			             possibleValues, possibleValues[2]);
			 if (selectedValue == 2 || selectedValue == -1) return;
			 if (selectedValue == 0) {
				 File f = this.m_indexPanel.getFilePanel().getImage();
				 if (f == null) return; // no image selected
				 this.m_indexPanel.save(f);
				 
				 TaggedImage img = DbHandler.getInstance().getImageFromPath(f);
				 if (img.isEmpty() == true) return; // no tags assigned

				 ArrayList<TaggedImage> images = new ArrayList<TaggedImage>();
				 images.add(img);
				 ServerCommunication.doUpload(images, this.m_indexPanel);
			 } else {
				 ServerCommunication.doUpload(null, this.m_indexPanel);
			 }
			System.out.println("up");
		} else if (sAcComm.equalsIgnoreCase("DOWNLOAD")) {
			// ask user if he want to upload only one or all images
			StringLib lib = StringLib.getInstance();
			 Object[] possibleValues = {lib.getValue("MainFrame.download.message.actImg"),
					 lib.getValue("MainFrame.download.message.allImg"), lib.getValue("MainFrame.download.message.cancel")};
			 int selectedValue = JOptionPane.showOptionDialog(this,
			             lib.getValue("MainFrame.download.message.text"), lib.getValue("MainFrame.download.message.title"),
			             JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null,
			             possibleValues, possibleValues[2]);
			 if (selectedValue == 2 || selectedValue == -1) return;
			 if (selectedValue == 0) {
				 File f = this.m_indexPanel.getFilePanel().getImage();
				 if (f == null || !f.exists()) return; // no image selected

				 TreeMap<String, ArrayList<File>> hashes = new TreeMap<String, ArrayList<File>>();
				 ArrayList<File> tmp = new ArrayList<File>();
				 tmp.add(f);
				 hashes.put(Hasher.imageToHash(f), tmp);
				 ServerCommunication.doDownload(hashes, this.m_indexPanel);
			 } else {
				 ServerCommunication.doDownload(null, this.m_indexPanel);
			 }
			System.out.println("dwn");
		} else if (sAcComm.equalsIgnoreCase("DBREORG")) {
			StringLib lib = StringLib.getInstance();
			int selectedValue = JOptionPane.showConfirmDialog(this,
		             lib.getValue("MainFrame.dbreorg.startmessage.text"), lib.getValue("MainFrame.dbreorg.startmessage.title"),
		             JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
			if (selectedValue != JOptionPane.YES_OPTION) return;
			
			// check if all images in database still exist
			TreeMap<File, Boolean> folders = DbHandler.getInstance().getDirectories();
	    	ArrayList<Integer> delIds = new ArrayList<Integer>();
	    	Iterator<File> folderIter = folders.keySet().iterator();
	    	while (folderIter.hasNext()) {
	    		File folder = folderIter.next();
	    		// check if the folder is on a removable media. if yes, skip it
	    		if (folders.get(folder)) continue;
	    		
	    		ArrayList<File> images = DbHandler.getInstance().getImagePathsOfDirectory(folder);
	    		
	    		for (File img : images) {
		    		if (!img.exists()) {
		    			delIds.add(DbHandler.getInstance().getImageIdFromPath(img));
		    		}
	    		}
	    	}
	    	
	    	if (delIds.size() != 0) {
				ArrayList<TaggedImage> alToRecreate = new ArrayList<TaggedImage>();
				for (int id : delIds) {
					alToRecreate.add(DbHandler.getInstance().getImageFromImageId(id));
				}
				Import.searchFilesForHashes(alToRecreate, this.m_indexPanel, null, false);
	    	} else {
	    		JOptionPane.showMessageDialog(this, lib.getValue("MainFrame.dbreorg.endmessage.text"), lib.getValue("MainFrame.dbreorg.endmessage.title"), JOptionPane.PLAIN_MESSAGE);
	    	}
	    	
			
		} else if (sAcComm.equalsIgnoreCase("INFO")) {
			new InfoDialog(this.m_indexPanel);
		} else {
			System.out.println("WTF?");
		}
		
	}

	/**
	 * @return the m_indexPanel
	 */
	public IndexPanel getIndexPanel() {
		return this.m_indexPanel;
	}
}