package search;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import allrounder.DbHandler;
import allrounder.StringLib;
import allrounder.TaggedImage;

public class ArtistSearchField extends JPanel implements SearchInterface, ActionListener {
	private static final long serialVersionUID = 8091731925749336310L;
	boolean bAddCloseButton = true;
	JButton btnClose = CloseIcon.getCloseButton();
	JLabel lblArtist = new JLabel(StringLib.getInstance().getValue("search.ArtistSearchField.artist"));
	JLabel lblOr = new JLabel(StringLib.getInstance().getValue("search.SearchInterface.or"));
	JComboBox jcbArtist;
	JCheckBox chkInvert = new JCheckBox(StringLib.getInstance().getValue("search.SearchInterface.invert"));
	Vector<ActionListener> listeners = new Vector<ActionListener>();
	
	public ArtistSearchField(boolean b, ActionListener al) {
		this.bAddCloseButton = b;
		this.listeners.add(al);
		construct();
	}
	
	public ArtistSearchField(ActionListener al) {
		this.listeners.add(al);
		construct();
	}
	
	public void addActionListener(ActionListener al) {
		this.listeners.add(al);
	}
	
	private void construct() {
		// get artists
		ArrayList<String> alArtists = DbHandler.getInstance().getArtists();
		Vector<String> vArtists = new Vector<String>();
		for (String s : alArtists) {
			vArtists.add(s);
		}
		java.util.Collections.sort(vArtists);
		this.jcbArtist = new JComboBox(vArtists);
		this.chkInvert.setBackground(Color.WHITE);
		
		//column / row
		double[][] layout = {{this.btnClose.getPreferredSize().width + 10, 10, this.lblOr.getPreferredSize().width, 10, this.lblArtist.getPreferredSize().width, 10, this.jcbArtist.getPreferredSize().width, 10, this.chkInvert.getPreferredSize().width},{TableLayoutConstants.FILL, this.jcbArtist.getPreferredSize().height, TableLayoutConstants.FILL}};
		this.setLayout(new TableLayout(layout));
		
		if (this.bAddCloseButton) {
			this.btnClose.setActionCommand("SUBCLOSE");
			this.btnClose.addActionListener(this);
			this.btnClose.setToolTipText(StringLib.getInstance().getValue("search.SearchCriteriaPanel.btnClose.tt"));
			add(this.btnClose, "0, 0, 0, 2, c, c");
			add(this.lblOr, "2, 0, 2, 2, c, c");
		}
		
		add(this.lblArtist, "4, 1");
		add(this.jcbArtist, "6, 1");
		add(this.chkInvert, "8, 1");
	}
	
	@Override
	public TreeSet<TaggedImage> filterImages(TreeSet<TaggedImage> images) {
		TreeSet<TaggedImage> resultSet = new TreeSet<TaggedImage>();
		Iterator<TaggedImage> it = images.iterator();
		while (it.hasNext()) {
			TaggedImage img = it.next();
			if (isValid(img)) {
				resultSet.add(img);
			}
		}
		
		return resultSet;
	}
	
	private boolean isValid(TaggedImage img) {
		if (this.jcbArtist.getSelectedItem() == null) return true;
		if (img.artists.contains(this.jcbArtist.getSelectedItem())) {
			return this.chkInvert.isSelected() ? false : true;
		}
		return this.chkInvert.isSelected() ? true : false;
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equalsIgnoreCase("SUBCLOSE")) {
			arg0.setSource(this);
			for (ActionListener al : this.listeners) {
				al.actionPerformed(arg0);
			}
		}
	}

}
