package search;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import allrounder.IdString;
import allrounder.StringLib;
import allrounder.TaggedImage;

public class GenderSearchField extends JPanel implements SearchInterface, ActionListener {
	private static final long serialVersionUID = -5829450979312372552L;
	boolean bAddCloseButton = true;
	JButton btnClose = CloseIcon.getCloseButton();
	JLabel lblSex = new JLabel(StringLib.getInstance().getValue("search.GenderSearchField.gender"));
	JLabel lblOr = new JLabel(StringLib.getInstance().getValue("search.SearchInterface.or"));
	JComboBox jcbSex;
	JCheckBox chkInvert = new JCheckBox(StringLib.getInstance().getValue("search.SearchInterface.invert"));
	Vector<ActionListener> listeners = new Vector<ActionListener>();
	
	public GenderSearchField(boolean b, ActionListener al) {
		this.bAddCloseButton = b;
		this.listeners.add(al);
		construct();
	}
	
	public GenderSearchField(ActionListener al) {
		this.listeners.add(al);
		construct();
	}
	
	public void addActionListener(ActionListener al) {
		this.listeners.add(al);
	}
	
	private void construct() {
		StringLib lib = StringLib.getInstance();
		
		// get Sexes
		Vector<IdString> vSex = new Vector<IdString>();
		int sexno = 0;
		while(true) {
			try {
				String tmp = lib.getValue("Tag.sex." + sexno + ".text");
				vSex.add(new IdString(sexno, tmp));
			} catch (IllegalArgumentException e) {
				// this WILL be thrown because i just iterate as long as there is no entry left
				break;
			}
			++sexno;
		}
		java.util.Collections.sort(vSex);
		this.jcbSex = new JComboBox(vSex);
		this.chkInvert.setBackground(Color.WHITE);
		
		//column / row
		double[][] layout = {{this.btnClose.getPreferredSize().width + 10, 10, this.lblOr.getPreferredSize().width, 10, this.lblSex.getPreferredSize().width, 10, this.jcbSex.getPreferredSize().width, 10, this.chkInvert.getPreferredSize().width},{TableLayoutConstants.FILL, this.jcbSex.getPreferredSize().height, TableLayoutConstants.FILL}};
		this.setLayout(new TableLayout(layout));
		
		if (this.bAddCloseButton) {
			this.btnClose.setActionCommand("SUBCLOSE");
			this.btnClose.addActionListener(this);
			this.btnClose.setToolTipText(StringLib.getInstance().getValue("search.SearchCriteriaPanel.btnClose.tt"));
			add(this.btnClose, "0, 0, 0, 2, c, c");
			add(this.lblOr, "2, 0, 2, 2, c, c");
		}
		
		add(this.lblSex, "4, 1");
		add(this.jcbSex, "6, 1");
		add(this.chkInvert, "8, 1");
	}
	
	@Override
	public TreeSet<TaggedImage> filterImages(TreeSet<TaggedImage> images) {
		TreeSet<TaggedImage> resultSet = new TreeSet<TaggedImage>();
		Iterator<TaggedImage> it = images.iterator();
		while (it.hasNext()) {
			TaggedImage img = it.next();
			if (isValid(img)) {
				resultSet.add(img);
			}
		}
		
		return resultSet;
	}
	
	private boolean isValid(TaggedImage img) {
		if (img.gender.contains(new Integer(((IdString)this.jcbSex.getSelectedItem()).getID()))) {
			return this.chkInvert.isSelected() ? false : true;
		}
		return this.chkInvert.isSelected() ? true : false;
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equalsIgnoreCase("SUBCLOSE")) {
			arg0.setSource(this);
			for (ActionListener al : this.listeners) {
				al.actionPerformed(arg0);
			}
		}
	}

}
