package search;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import allrounder.StringLib;
import allrounder.TaggedImage;

public class HeadCountSearchField extends JPanel implements SearchInterface, ActionListener, FocusListener {
	private static final long serialVersionUID = -8982715956531576977L;
	boolean bAddCloseButton = true;
	JButton btnClose = CloseIcon.getCloseButton();
	JLabel lblCount = new JLabel(StringLib.getInstance().getValue("search.HeadCountSearchField.count"));
	JLabel lblOr = new JLabel(StringLib.getInstance().getValue("search.SearchInterface.or"));
	JComboBox jcbRangebox;
	JTextField jtfHeadcount = new JTextField("0");
	Vector<ActionListener> listeners = new Vector<ActionListener>();
	
	public HeadCountSearchField(boolean b, ActionListener al) {
		this.bAddCloseButton = b;
		this.listeners.add(al);
		construct();
	}
	
	public HeadCountSearchField(ActionListener al) {
		this.listeners.add(al);
		construct();
	}
	
	public void addActionListener(ActionListener al) {
		this.listeners.add(al);
	}
	
	private void construct() {
		String[] range = {"<=", "=", ">="};
		this.jcbRangebox = new JComboBox(range);
		this.jtfHeadcount.addFocusListener(this);
		
		//column / row
		double[][] layout = {{this.btnClose.getPreferredSize().width + 10, 10, this.lblOr.getPreferredSize().width, 10, this.lblCount.getPreferredSize().width, 10, this.jcbRangebox.getPreferredSize().width, 10, TableLayoutConstants.FILL},{TableLayoutConstants.FILL, this.jcbRangebox.getPreferredSize().height, TableLayoutConstants.FILL}};
		this.setLayout(new TableLayout(layout));
		
		if (this.bAddCloseButton) {
			this.btnClose.setActionCommand("SUBCLOSE");
			this.btnClose.addActionListener(this);
			this.btnClose.setToolTipText(StringLib.getInstance().getValue("search.SearchCriteriaPanel.btnClose.tt"));
			add(this.btnClose, "0, 0, 0, 2, c, c");
			add(this.lblOr, "2, 0, 2, 2, c, c");
		}
		
		add(this.lblCount, "4, 1");
		add(this.jcbRangebox, "6, 1");
		add(this.jtfHeadcount, "8, 1");
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				HeadCountSearchField.this.jtfHeadcount.requestFocusInWindow();
				HeadCountSearchField.this.jtfHeadcount.selectAll();
			}
		});
	}
	
	@Override
	public TreeSet<TaggedImage> filterImages(TreeSet<TaggedImage> images) {
		TreeSet<TaggedImage> resultSet = new TreeSet<TaggedImage>();
		Iterator<TaggedImage> it = images.iterator();
		while (it.hasNext()) {
			TaggedImage img = it.next();
			if (isValid(img)) {
				resultSet.add(img);
			}
		}
		
		return resultSet;
	}
	
	private boolean isValid(TaggedImage img) {
		String s = (String)this.jcbRangebox.getSelectedItem();
		int headcount = 0;
		try {
			headcount = Integer.parseInt(this.jtfHeadcount.getText());
		} catch(NumberFormatException e) {
			this.jtfHeadcount.setText("0");
			return false;
		}
		
		if (s.equals("<=")) {
			if (img.headcount <= headcount) return true;
		} else if (s.equals("=")) {
			if (img.headcount == headcount) return true;
		} else if (s.equals(">=")) {
			if (img.headcount >= headcount) return true;
		}
		return false;
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equalsIgnoreCase("SUBCLOSE")) {
			arg0.setSource(this);
			for (ActionListener al : this.listeners) {
				al.actionPerformed(arg0);
			}
		}
	}

	@Override
	public void focusGained(FocusEvent arg0) {}

	@Override
	public void focusLost(FocusEvent arg0) {
		try {
			Integer.parseInt(this.jtfHeadcount.getText());
		} catch(NumberFormatException e) {
			this.jtfHeadcount.setText("0");
		}
	}

}
