package search;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.SwingConstants;

import allrounder.IdString;
import allrounder.StringLib;
import allrounder.TaggedImage;

public class SearchCriteriaPanel extends JPanel implements SearchInterface, ActionListener {
	private static final long serialVersionUID = -5969811177508456502L;
	StringLib lib = StringLib.getInstance();
	boolean bAddCloseButton = true;
	JButton btnClose = CloseIcon.getCloseButton();
	JLabel lblCategory = new JLabel(this.lib.getValue("search.SearchCriteriaPanel.category"));
	JComboBox jcbCategories;
	JButton btnAddAlternative = new JButton(this.lib.getValue("search.SearchCriteriaPanel.btnAddAlternative.text"));
	Vector<ActionListener> listeners = new Vector<ActionListener>();
	JPanel pnlSearchfields = new JPanel();
	Vector<SearchInterface> vectSearchfields = new Vector<SearchInterface>();
	
	public SearchCriteriaPanel(boolean b, ActionListener al) {
		this.bAddCloseButton = b;
		this.listeners.add(al);
		construct();
	}
	
	public SearchCriteriaPanel(ActionListener al) {
		this.listeners.add(al);
		construct();
	}
	
	public void addActionListener(ActionListener al) {
		this.listeners.add(al);
	}
	
	private void construct() {
		setBackground(Color.WHITE);
		
		Vector<IdString> vect = new Vector<IdString>();

		//vect.add(new IdString(0, "---"));
		int i = 1;
		while(true) {
			try {
				vect.add(new IdString(i, this.lib.getValue("search.SearchCriteriaPanel.category." + i)));
			} catch (IllegalArgumentException e) {
				// this WILL be thrown because i just iterate as long as there is no entry left
				break;
			}
			++i;
		}
		java.util.Collections.sort(vect);
		this.jcbCategories = new JComboBox(vect);
		this.jcbCategories.addActionListener(this);
		this.btnAddAlternative.setToolTipText(this.lib.getValue("search.SearchCriteriaPanel.btnAddAlternative.tt"));
		this.btnAddAlternative.setActionCommand("ADDALTER");
		this.btnAddAlternative.addActionListener(this);
		this.pnlSearchfields.setBackground(Color.WHITE);
		this.pnlSearchfields.setLayout(new BoxLayout(this.pnlSearchfields, BoxLayout.Y_AXIS));
		
		//column / row
		double[][] layout = {{this.btnClose.getPreferredSize().width + 10, 10, this.jcbCategories.getPreferredSize().width, 10, TableLayoutConstants.FILL, 10, this.btnAddAlternative.getPreferredSize().width},{this.lblCategory.getPreferredSize().height, this.jcbCategories.getPreferredSize().height, 30, 2}};
		this.setLayout(new TableLayout(layout));
		
		if (this.bAddCloseButton) {
			this.btnClose.setActionCommand("CLOSE");
			this.btnClose.addActionListener(this);
			this.btnClose.setToolTipText(this.lib.getValue("search.SearchCriteriaPanel.btnClose.tt"));
			this.add(this.btnClose, "0, 0, 0, 2, c, c");
		}
		
		add(this.lblCategory, "2, 0");
		add(this.jcbCategories, "2, 1");
		add(this.pnlSearchfields, "4, 0, 4, 2");
		add(this.btnAddAlternative, "6, 0, 6, 2, c, c");
		add(new JSeparator(SwingConstants.HORIZONTAL), "0, 3, 6, 3");
		
		setMaximumSize(new Dimension(this.getMaximumSize().width, this.getPreferredSize().height));
		
		// set default ctgory (free text)
		this.jcbCategories.setSelectedItem(new IdString(8, this.lib.getValue("search.SearchCriteriaPanel.category." + 8)));
	}

	@Override
	public TreeSet<TaggedImage> filterImages(TreeSet<TaggedImage> images) {
		TreeSet<TaggedImage> resultImages = new TreeSet<TaggedImage>();
		for (SearchInterface si : this.vectSearchfields) {
			resultImages.addAll(si.filterImages(images));
		}
		return resultImages;
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equalsIgnoreCase("ADDALTER")) {
			alternativeAddOrComboBoxChanged(false);
		} else if (arg0.getActionCommand().equalsIgnoreCase("CLOSE")) {
			arg0.setSource(this);
			for (ActionListener al : this.listeners) {
				al.actionPerformed(arg0);
			}
		} else if (arg0.getActionCommand().equalsIgnoreCase("comboBoxChanged")) {
			alternativeAddOrComboBoxChanged(true);
		} else if (arg0.getActionCommand().equalsIgnoreCase("SUBCLOSE")) {
			this.vectSearchfields.remove(arg0.getSource());
			buildSearchfieldPanel();
		}
	}
	
	private void alternativeAddOrComboBoxChanged(boolean bComboBoxChanged) {
		int id = ((IdString)this.jcbCategories.getSelectedItem()).getID();
		if (bComboBoxChanged) this.vectSearchfields.removeAllElements();
		boolean bCloseButton = true;
		if (bComboBoxChanged) bCloseButton = false;
		switch(id) {
			case 1:
				// headcount
				this.vectSearchfields.add(new HeadCountSearchField(bCloseButton, this));
				break;
			case 2:
				// artist
				this.vectSearchfields.add(new ArtistSearchField(bCloseButton, this));
				break;
			case 3:
				// spezies
				this.vectSearchfields.add(new SpeziesSearchField(bCloseButton, this));
				break;
			case 4:
				// other
				this.vectSearchfields.add(new OtherSearchField(bCloseButton, this));
				break;
			case 5:
				// resolution
				this.vectSearchfields.add(new ResolutionSearchField(bCloseButton, this));
				break;
			case 6:
				// fetish
				this.vectSearchfields.add(new YiffSearchField(bCloseButton, this));
				break;
			case 7:
				// gender
				this.vectSearchfields.add(new GenderSearchField(bCloseButton, this));
				break;
			case 8:
				// freetext
				this.vectSearchfields.add(new TextInputSearchField(bCloseButton, this));
				break;
			default:
				this.vectSearchfields.removeAllElements();
				break;
		}
		buildSearchfieldPanel();
	}
	
	private void buildSearchfieldPanel() {
		this.pnlSearchfields.removeAll();
		//pnlSearchfields.add(Box.createVerticalGlue());
		for (SearchInterface pnl : this.vectSearchfields) {
			if (!(pnl instanceof JPanel)) continue;
			((JPanel)pnl).setBackground(Color.WHITE);
			this.pnlSearchfields.add((JPanel)pnl);
		}
		
		this.pnlSearchfields.revalidate();
		this.pnlSearchfields.repaint();
		if (getPreferredSize().height < this.pnlSearchfields.getPreferredSize().height) {
			// get the size: absolute height - rows 0, 1
			int size = this.pnlSearchfields.getPreferredSize().height - this.lblCategory.getPreferredSize().height - this.jcbCategories.getPreferredSize().height;
			((TableLayout)this.getLayout()).setRow(2, size);
			this.setMaximumSize(new Dimension(getMaximumSize().width, getPreferredSize().height));
		} else {
			((TableLayout)this.getLayout()).setRow(2, this.pnlSearchfields.getPreferredSize().height);
			this.setMaximumSize(new Dimension(getMaximumSize().width, getPreferredSize().height));
		}
	}
}
