package search;

import index.ImagePanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.event.MouseInputAdapter;

public class SlideShow extends JFrame {
	ArrayList<File> alFiles;
	WaiterThread waiter;
	ImgAddrGlassPane glassPane;
	ImageIcon imgIco = new ImageIcon();
	Image prevImg = null;
	int actualIndex = 0;

	public SlideShow(ArrayList<File> alImgs) {
		if (alImgs == null || alImgs.size() == 0) return;
		this.alFiles = alImgs;
		
		this.addMouseListener(new MouseInputAdapter() {
			@Override
			public void mouseReleased(MouseEvent e) {
				// also catches drag events, not just clicks
				displayNextImg();
			}
		});
		this.addKeyListener(new KeyAdapter() {
			@Override
			public void keyPressed(KeyEvent arg0) {
				if (arg0.getKeyCode() == KeyEvent.VK_ESCAPE) {
					SlideShow.this.waiter.stop();
					SlideShow.this.waiter = null;
					GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().setFullScreenWindow(null);
					SlideShow.this.setVisible(false);
					SlideShow.this.dispose();
				} else if(arg0.getKeyCode() == KeyEvent.VK_RIGHT) {
					displayNextImg();
				} else if(arg0.getKeyCode() == KeyEvent.VK_LEFT) {
					displayPrevImg();
				}
			}
		});
		
		this.setUndecorated(true);

		// load and display first image
		this.imgIco.setImage(loadScaled(this.alFiles.get(0)));
		
		JLabel lbl = new JLabel(this.imgIco);
		lbl.setPreferredSize(this.getSize());
		lbl.setBackground(Color.BLACK);
		lbl.setOpaque(true);
		this.add(lbl, BorderLayout.CENTER);
		
		// the following will work fine on linux (where the full screen window is only simulated), but on windows,
		// it displays the content of the search dialog over it (until a repaint some time later or so, dunno)
		// GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().setFullScreenWindow(this);
		// instead, I'll do it manually
		this.setAlwaysOnTop(true);
		this.setBounds(0, 0, Toolkit.getDefaultToolkit().getScreenSize().width, Toolkit.getDefaultToolkit().getScreenSize().height);
		
		// load prev img
		if (this.alFiles.size() != 1) this.prevImg = loadScaled(this.alFiles.get(this.alFiles.size() - 1));
		this.glassPane = new ImgAddrGlassPane(this.alFiles.get(0).getAbsolutePath(), this);
		this.setGlassPane(this.glassPane);
		
		this.setVisible(true);
		this.glassPane.setVisible(true);
		
		this.waiter = new WaiterThread(this);
		new Thread(this.waiter).start();
		
		//this.repaint();
		//glassPane.repaint();
	}
	
	protected void displayNextImg() {
		this.waiter.doNothing(true);
		if (this.alFiles.size() == 1) return;
		++this.actualIndex;
		if (this.actualIndex >= this.alFiles.size()) this.actualIndex = 0;
		
		// kill previous prevImg
		this.prevImg.flush();
		
		this.prevImg = this.imgIco.getImage();
		this.imgIco.setImage(loadScaled(this.alFiles.get(this.actualIndex)));
		repaint();
		this.glassPane.setString(this.alFiles.get(this.actualIndex).getAbsolutePath());
		this.waiter.resetTimer();
		this.waiter.doNothing(false);
	}
	
	protected void displayPrevImg() {
		this.waiter.doNothing(true);
		if (this.alFiles.size() == 1) return;
		--this.actualIndex;
		if (this.actualIndex < 0) this.actualIndex = this.alFiles.size() - 1;
		this.imgIco.setImage(this.prevImg);
		
		// kill previous prevImg
		this.prevImg.flush();
		if (this.actualIndex == 0) {
			this.prevImg = loadScaled(this.alFiles.get(this.alFiles.size() - 1));
		} else {
			this.prevImg = loadScaled(this.alFiles.get(this.actualIndex - 1));
		}
		repaint();
		this.glassPane.setString(this.alFiles.get(this.actualIndex).getAbsolutePath());
		this.waiter.resetTimer();
		this.waiter.doNothing(false);
	}
	
	private Image loadScaled(File fImageFile) {
		Image img = ImagePanel.readImgFromFile(fImageFile, this);
		return ImagePanel.scaleToFit(img, Toolkit.getDefaultToolkit().getScreenSize(), Image.SCALE_DEFAULT, this);
	}
}

class ImgAddrGlassPane extends JComponent {
	String theString = "";
	JFrame parent;
	
	public ImgAddrGlassPane(String str, JFrame parentFrame) {
		this.parent = parentFrame;
		this.theString = str;
	}
	
	@Override
	public void paint(Graphics g){
		int x = 25;
		int y = 25;
		g.setColor(Color.white);
		Font f = new Font("Times",Font.PLAIN,18);
		g.setFont(f);
		// resize the string so the file name (or, for extraordinary long file names, part of it) is always visible
		while (g.getFontMetrics(f).stringWidth(this.theString) + x > this.parent.getSize().width) {
			this.theString = "..." + this.theString.substring(4);
		}
		g.drawString(this.theString, x - 1, y);
		g.drawString(this.theString, x + 1, y);
		g.drawString(this.theString, x, y - 1);
		g.drawString(this.theString, x, y + 1);
		 
		g.setColor(Color.black);
		g.drawString(this.theString, x, y);
	}
	
	public void setString(String str) {
		this.theString = str;
		repaint();
	}

}


class WaiterThread implements Runnable {
	SlideShow sShow = null;
	int iTimeWaited = 0;
	boolean bStop = false;
	boolean bSkip = false;
	
	public WaiterThread(SlideShow s) {
		this.sShow = s;
	}

	public void run() {
		while (true) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			if (this.bStop) break;
			if (this.bSkip) continue;
			this.iTimeWaited += 100;
			if (this.iTimeWaited >= 4000) {
				this.sShow.displayNextImg();
				this.iTimeWaited = 0;
			}
		}
	}
	
	public void resetTimer() {
		this.iTimeWaited = 0;
	}
	
	public void stop() {
		this.bStop = true;
	}
	
	public void doNothing(boolean b) {
		this.bSkip = b;
	}
}