package search;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import allrounder.IdString;
import allrounder.StringLib;
import allrounder.TaggedImage;

public class SpeziesSearchField extends JPanel implements SearchInterface, ActionListener {
	private static final long serialVersionUID = 7231071507531970242L;
	StringLib lib = StringLib.getInstance();
	boolean bAddCloseButton = true;
	JButton btnClose = CloseIcon.getCloseButton();
	JLabel lblSpezies = new JLabel(StringLib.getInstance().getValue("search.SpeziesSearchField.spezies"));
	JLabel lblOr = new JLabel(StringLib.getInstance().getValue("search.SearchInterface.or"));
	JComboBox jcbSpezies;
	JCheckBox chkInvert = new JCheckBox(StringLib.getInstance().getValue("search.SearchInterface.invert"));
	Vector<ActionListener> listeners = new Vector<ActionListener>();
	
	public SpeziesSearchField(boolean b, ActionListener al) {
		this.bAddCloseButton = b;
		this.listeners.add(al);
		construct();
	}
	
	public SpeziesSearchField(ActionListener al) {
		this.listeners.add(al);
		construct();
	}
	
	public void addActionListener(ActionListener al) {
		this.listeners.add(al);
	}
	
	private void construct() {		
		// get spezies
		Vector<IdString> vSpezies = new Vector<IdString>();
		String tmp = this.lib.getValue("0"); // comma separated list of all spezies
		String[] speziesArr = tmp.split(",");
		for (String s : speziesArr) {
			IdString spezies = new IdString(Integer.parseInt(s), this.lib.getValue("species." + s));
			vSpezies.add(spezies);
		}
		
		java.util.Collections.sort(vSpezies);
		this.jcbSpezies = new JComboBox(vSpezies);
		this.chkInvert.setBackground(Color.WHITE);
		
		//column / row
		double[][] layout = {{this.btnClose.getPreferredSize().width + 10, 10, this.lblOr.getPreferredSize().width, 10, this.lblSpezies.getPreferredSize().width, 10, this.jcbSpezies.getPreferredSize().width, 10, this.chkInvert.getPreferredSize().width},{TableLayoutConstants.FILL, this.jcbSpezies.getPreferredSize().height, TableLayoutConstants.FILL}};
		this.setLayout(new TableLayout(layout));
		
		if (this.bAddCloseButton) {
			this.btnClose.setActionCommand("SUBCLOSE");
			this.btnClose.addActionListener(this);
			this.btnClose.setToolTipText(StringLib.getInstance().getValue("search.SearchCriteriaPanel.btnClose.tt"));
			add(this.btnClose, "0, 0, 0, 2, c, c");
			add(this.lblOr, "2, 0, 2, 2, c, c");
		}
		
		add(this.lblSpezies, "4, 1");
		add(this.jcbSpezies, "6, 1");
		add(this.chkInvert, "8, 1");
	}
	
	@Override
	public TreeSet<TaggedImage> filterImages(TreeSet<TaggedImage> images) {
		TreeSet<TaggedImage> resultSet = new TreeSet<TaggedImage>();
		Iterator<TaggedImage> it = images.iterator();
		while (it.hasNext()) {
			TaggedImage img = it.next();
			if (isValid(img)) {
				resultSet.add(img);
			}
		}
		
		return resultSet;
	}
	
	private boolean isValid(TaggedImage img) {
		int id = ((IdString)this.jcbSpezies.getSelectedItem()).getID();
		if (id >= 0) {
			if (img.spezies.contains(id)) return !this.chkInvert.isSelected();
		} else {
			// get possible spezies
			String tmp = this.lib.getValue((id * -1) + ""); // comma separated list of all spezies of which this -id belongs to
			String[] speziesArr = tmp.split(",");
			for (String s : speziesArr) {
				if (img.spezies.contains(Integer.parseInt(s))) {
					return this.chkInvert.isSelected() ? false : true;
				}
			}
		}
		return this.chkInvert.isSelected() ? true : false;
	}

	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getActionCommand().equalsIgnoreCase("SUBCLOSE")) {
			arg0.setSource(this);
			for (ActionListener al : this.listeners) {
				al.actionPerformed(arg0);
			}
		}
	}

}
