package search;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import allrounder.StringLib;
import allrounder.TaggedImage;

public class TextInputSearchField  extends JPanel implements SearchInterface, ActionListener {
	boolean bAddCloseButton = true;
	JButton btnClose = CloseIcon.getCloseButton();
	JLabel lblOr = new JLabel(StringLib.getInstance().getValue("search.SearchInterface.or"));
	JTextField jtfText = new JTextField();
	Vector<ActionListener> listeners = new Vector<ActionListener>();
	
	public TextInputSearchField(boolean b, ActionListener al) {
		this.bAddCloseButton = b;
		this.listeners.add(al);
		construct();
	}
	
	public TextInputSearchField(ActionListener al) {
		this.listeners.add(al);
		construct();
	}
	
	private void construct() {
		//column / row
		double[][] layout = {{this.btnClose.getPreferredSize().width + 10, 10, this.lblOr.getPreferredSize().width, 10, TableLayoutConstants.FILL},{TableLayoutConstants.FILL, this.jtfText.getPreferredSize().height, TableLayoutConstants.FILL}};
		this.setLayout(new TableLayout(layout));
		
		if (this.bAddCloseButton) {
			this.btnClose.setActionCommand("SUBCLOSE");
			this.btnClose.addActionListener(this);
			this.btnClose.setToolTipText(StringLib.getInstance().getValue("search.SearchCriteriaPanel.btnClose.tt"));
			add(this.btnClose, "0, 1, c, c");
			add(this.lblOr, "2, 1, c, c");
		}
		
		add(this.jtfText, "4, 1");
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				TextInputSearchField.this.jtfText.requestFocusInWindow();
			}
		});
	}
	
	public TreeSet<TaggedImage> filterImages(TreeSet<TaggedImage> images) {
		TreeSet<TaggedImage> resultImages = new TreeSet<TaggedImage>();
		if (this.jtfText.getText().trim().equals("")) return resultImages;
		String[] searchWords = this.jtfText.getText().trim().split(" ");
		
		// get species names
		String[] sAllSpeciesIds = StringLib.getInstance().getValue("0").split(",");
		TreeMap<String, Integer> speziMap = new TreeMap<String, Integer>(String.CASE_INSENSITIVE_ORDER);
		for (String speciesID : sAllSpeciesIds) {
			speziMap.put(StringLib.getInstance().getValue("species." + speciesID), Integer.parseInt(speciesID));
		}
		
		// get fetishes
		int fetishno = 0;
		TreeMap<String, Integer> fetishMap = new TreeMap<String, Integer>(String.CASE_INSENSITIVE_ORDER);
		while(true) {
			String tempText = null;
			try {
				tempText = StringLib.getInstance().getValue("fetishchooser.fetishes." + fetishno + ".text");
			} catch (IllegalArgumentException e) {
				// this WILL be thrown because i just iterate as long as there is no entry left
			}
			if (tempText == null) break;
			fetishMap.put(tempText, fetishno);
			++fetishno;
		}
		
		// get genders
		int sexno = 0;
		TreeMap<String, Integer> genderMap = new TreeMap<String, Integer>(String.CASE_INSENSITIVE_ORDER);
		while(true) {
			String tempText = null;
			try {
				tempText = StringLib.getInstance().getValue("Tag.sex." + sexno + ".text");
			} catch (IllegalArgumentException e) {
				// this WILL be thrown because i just iterate as long as there is no entry left
			}
			if (tempText == null) break;
			genderMap.put(tempText, sexno);
			++sexno;
		}
		
		for (TaggedImage img : images) {
			if (isValid(img, searchWords, speziMap, fetishMap, genderMap)) {
				resultImages.add(img);
			}
		}
		
		return resultImages;
	}
	
	private boolean isValid(TaggedImage img, String[] searchWords, TreeMap<String, Integer> speciesMap,
			TreeMap<String, Integer> fetishMap, TreeMap<String, Integer> genderMap) {
		for (String searchWord : searchWords) {
			// yiff (w/o fetish)
			if (searchWord.equalsIgnoreCase("yiff") && img.yiff) continue;
			
			// filename
			if (img.path.getName().toLowerCase().contains(searchWord.toLowerCase())) {
				continue;
			}
			
			// artist
			boolean bContinue = false;
			for (String s : img.artists) {
				if (s.equalsIgnoreCase(searchWord)) {
					bContinue = true;
					break;
				}
			}
			if (bContinue) continue;
			
			// spezies
			if (speciesMap.containsKey(searchWord) && img.spezies.contains(speciesMap.get(searchWord))) {
				continue;
			}
			
			// fetish
			Iterator<String> iter = fetishMap.keySet().iterator();
			while (iter.hasNext()) {
				String fetish = iter.next();
				if (!fetish.toLowerCase().contains(searchWord)) continue;
				// word could be a fetish
				if (img.fetish.contains(fetishMap.get(fetish))) {
					bContinue = true;
					break;
				}
			}
			if (bContinue) continue;
			
			// other tags
			for (String tag : img.other) {
				if (tag.toLowerCase().contains(searchWord.toLowerCase())) {
					bContinue = true;
					break;
				}
			}
			if (bContinue) continue;
			
			// gender
			iter = genderMap.keySet().iterator();
			while (iter.hasNext()) {
				String gender = iter.next();
				if (!gender.toLowerCase().contains(searchWord)) continue;
				// word could be a fetish
				if (img.gender.contains(genderMap.get(gender))) {
					bContinue = true;
					break;
				}
			}
			if (bContinue) continue;
			
			// rest of the tags are all numbers
			int searchInt = 0;
			try {
				searchInt = Integer.parseInt(searchWord);
			} catch (NumberFormatException e) {
				return false;
			}
			
			// headcount
			if (img.headcount == searchInt) continue;
			
			// resolution
			if (img.width == searchInt || img.height == searchInt) continue;
			
			// if it reaches this, the actual search word matched no field of this image, abort
			return false;
		}
		
		return true;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equalsIgnoreCase("SUBCLOSE")) {
			e.setSource(this);
			for (ActionListener al : this.listeners) {
				al.actionPerformed(e);
			}
		}
	}

}
